from __future__ import absolute_import
from __future__ import unicode_literals

from laipvt.interface.serviceinterface import ServiceInterface
from laipvt.sysutil.util import status_me


class NlpLaYouTlmController(ServiceInterface):
	def __init__(self, check_result, service_path):
		super(NlpLaYouTlmController, self).__init__(check_result, service_path)
		self.tf_name = "nlp-layoutlm"

	@status_me("nlp_layoutlm")
	def push_nlp_layoutlm_images(self):
		self.push_images(self.project)

	@status_me("nlp_layoutlm")
	def start_nlp_layoutlm_service(self):
		self._create_namespace(namespaces=self.namespaces,
		                       istio_injection_namespaces=self.istio_injection_namespaces)
		self.start_service(project=self.project, version=self.private_deploy_version, tf_name=self.tf_name)

	@status_me("nlp_layoutlm")
	def prepare_nlp_layoutlm_data(self):
		self.prepare_data(project=self.project)

	def run(self):
		self.push_nlp_layoutlm_images()
		self.start_nlp_layoutlm_service()
		self.prepare_nlp_layoutlm_data()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#!/usr/bin/env python
# -*- encoding: utf-8 -*-
import os
import json
from laipvt.helper.errors import Helper
from laipvt.sysutil.gvalue import CHECK_FILE, PROJECT_INFO_FILE
from laipvt.sysutil.args_poc import Args
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.handler.packagehandler import DeployPackageHandler
from laipvt.controller.kubernetes.kube import KubeController
from laipvt.controller.middleware.harbor import HarborController
from laipvt.controller.middleware.nginx import NginxController
from laipvt.controller.middleware.etcd import EtcdController
from laipvt.controller.middleware.minio import MinioController
from laipvt.controller.middleware.redis import RedisController
from laipvt.controller.middleware.mysql import MysqlController
from laipvt.controller.middleware.elasticsearch import EsController
from laipvt.controller.middleware.rabbitmq import RabbitmqController
from laipvt.controller.middleware.identity import IdentityController
from laipvt.controller.middleware.siber import SiberController
from laipvt.controller.service.license import LicenseController
from laipvt.controller.service.mage import MageController
from laipvt.controller.service.ocr_standard import OcrStandardController
from laipvt.controller.service.nlp import NlpController
from laipvt.controller.service.seal import SealController
from laipvt.controller.service.captcha import CaptchaController
from laipvt.controller.service.commander import CommanderController
from laipvt.controller.service.ocr import OcrController
from laipvt.controller.service.chatbot import ChatbotController
from laipvt.controller.service.usercenter import UserCenterController
from laipvt.controller.service.dataservice import DataServiceController
from laipvt.controller.service.ocr_text import OcrTexttableController
from laipvt.controller.service.ocr_idcard_server import OcrIdcardController
from laipvt.controller.middleware.monitor import MonitorController
from laipvt.controller.middleware.keepalived import KeepalivedController
from laipvt.handler.middlewarehandler import EtcdConfigHandler, MysqlConfigHandler, EsConfigHandler, \
    MinioConfigHandler, RabbitmqConfigHandler, RedisConfigHandler, HarborConfigHandler, NginxConfigHandler, \
    IdentityConfigHandler, SiberConfigHandler, OcrConfigHandler, MonitorConfigHandler, KeepalivedConfigHandler
from laipvt.sysutil.util import find, write_to_file, read_form_json_file, gen_https_self_signed_ca, modify_umask
from laipvt.sysutil.kube_common import wait_pod_running
from laipvt.sysutil.check import is_pre_check, check_use_https_ca, check_https_ca_self_signed




# def main():
#     print("test")

def main():
    args = Args().parse_args()

    if not args.isCheck:
        if not is_pre_check():
            print(Helper().PRECHECK_FAILED)
            exit(2)

    # 获取前置检查结果
    check_result_file = CHECK_FILE
    check_result = CheckResultHandler(check_result_file)

    # 设置umask
    modify_umask(check_result=check_result, is_set=True)

    if args.targzFile:
        pkg_path = False
        if not os.path.exists(args.targzFile):
            cwd = [os.getcwd(), check_result.deploy_dir]
            for d in cwd:
                pkg_path = find(d, args.targzFile, file=True)
                if pkg_path:
                    break
        else:
            pkg_path = os.path.join(os.getcwd(), args.targzFile)
        if not pkg_path:
            print(Helper().FILE_NOT_FOUND)
            exit(2)
        PKG = os.path.dirname(pkg_path)
        ID = os.path.basename(pkg_path).split(".")[0]
        PKG_DIR = pkg_path.split(".")[0]

        # 将项目ID和path写入文件缓存
        project_dict = { "PKG": PKG, "ID": ID }
        write_to_file(PROJECT_INFO_FILE, json.dumps(project_dict, indent=4))

        deploy_package = DeployPackageHandler(PKG, ID)
        if not os.path.exists(PKG_DIR):
            deploy_package.unpack()
        # 解析
        parse_package = deploy_package.parse()
        kubernetes_package = parse_package.kubernetes
        middleware_package = parse_package.middleware
        harbor_package = parse_package.harbor

        if not os.path.exists(os.path.join(PKG_DIR, "middleware")):
            middleware_package.unpack()
        if not os.path.exists(os.path.join(PKG_DIR, "harbor")):
            harbor_package.unpack()

        # install harbor
        haror_path = harbor_package.parse().harbor
        harbor_config = HarborConfigHandler()
        harbor = HarborController(check_result, harbor_config, haror_path)
        harbor.install_harbor()

        # install nginx
        nginx_package = middleware_package.parse().nginx
        nginx_config = NginxConfigHandler()
        nginx = NginxController(check_result, nginx_config, nginx_package)
        nginx.install_nginx()


        # # install identity
        # identity_path = middleware_package.parse().identity
        # identity_config = IdentityConfigHandler()
        # identity = IdentityController(check_result, identity_config, identity_path)
        # identity.deploy_identity()



from __future__ import absolute_import
from __future__ import unicode_literals

import os
from laipvt.sysutil.util import path_join, run_local_cmd, to_object, status_me
from laipvt.helper.exception import Forbidden, HandlerError
from laipvt.helper.errors import PackageErrors, Error
from laipvt.handler.confighandler import PvtAdminConfigHandler, ServiceConfigHandler


"""
        目录结构
        授权id.tar.gz
          |- pvtadmin.json
          |- kubernetes.tar.gz
            |- templates/
              |- admin.tmpl
            |- plugin/
              |- network/
                  |- flannel.yaml
                  |- calico.yaml
              |- helm/
                  |- bin/
              |- istio/
                  |- bin/
                  |- istio.yaml
          |- middleware.tar.gz
            |- mysql/
                |- xxx.tmpl (docker-compose文件)
                |- nginx-xxx.tmpl
                |- config.tmpl
            |- redis/
            |- ...
          |- harbor.tar.gz
            |- install.sh
          |- mage.tar.gz
            |- config.yaml
            |- init/
              |- mysql/
            |- images/
              |- xxx.tar
            |- templates/
              |- xxx.conf
            |- charts/
          |- nlp.tar.gz
            |- config.yaml
            |- images
              |- xxx.tar
            |- data
              |- xxx
"""


class PackageHandler(object):
    def __init__(self, path: str, project_name: str):
        # 存放包的目录
        self.location = path
        self.project_name = project_name
        self.package_name = "{}.tar.gz".format(self.project_name)
        # 要部署的项目名称
        self.root_dir = path_join(self.location, self.project_name)
        self.pkg = path_join(self.location, self.package_name)

    def unpack(self, nozip=False, package_name=None) -> bool:
        # print(package_name)
        if package_name:
            self.package_name = package_name
            self.root_dir = path_join(self.location, package_name.split(".tar.gz")[0])
            # 如果已经存在解压的目录则跳过解压步骤
            if os.path.exists(self.root_dir):
                return True
        cmd = "tar zxf {} -C {}".format(path_join(self.location, self.package_name), self.location)
        if nozip:
            cmd = "tar xf {} -C {}".format(path_join(self.location, self.package_name), self.location)
        res = run_local_cmd(cmd)
        if res.code == 0 and os.path.isdir(self.root_dir):
            return True
        raise HandlerError(PackageErrors().UNPACK_ERROR.format(res.stdout)) from None

    def pack(self, nozip=False) -> bool:
        cmd = "cd {} && tar zcvf {} {}".format(self.location, self.package_name, self.project_name)
        if nozip:
            cmd = "cd {} && tar cvf {} {}".format(self.location, self.package_name, self.project_name)
        res = run_local_cmd(cmd)
        if res.code == 0 and os.path.isdir(self.root_dir):
            return self.pkg
        raise HandlerError(PackageErrors().PACK_ERROR.format(res.stdout)) from None

    def must_in(self, name: str, path: str) -> str:
        if name in os.listdir(path):
            return path_join(path, name)
        raise Forbidden(PackageErrors().PACKAGE_ILLEGAL.format(name)) from None

    def parse(self):
        raise HandlerError(Error().UNIMPLEMENT_ERROR.format("PackageHandler.parse"))

    def get(self):
        self.unpack()
        return self.parse()


class DeployPackageHandler(PackageHandler):
    """
    解析abcdefghi.tar.gz
    """
    def __init__(self, path: str, apply_id: str):
        super(DeployPackageHandler, self).__init__(path, apply_id)
        # if len(apply_id) != 9:
        #     raise Forbidden(PackageErrors().PROJECT_ID_INCORRECT)

    def unpack(self):
        super().unpack(nozip=True)

    def parse(self):
        pvtadmin_file = "config.json"
        for i in os.listdir(self.root_dir):
            if pvtadmin_file in i:
                pvtadmin_file = i
        admin_config = PvtAdminConfigHandler(path_join(self.root_dir, pvtadmin_file))
        deploy_projects = admin_config.service_list.get()
        service_list = []
        for s in deploy_projects:
            if s == "ocr_3rd":
                service_list.append(OcrPackageHandler(self.root_dir, s))
            else:
                service_list.append(ServicePackageHandler(self.root_dir, s))
        res = {
            "config": admin_config,
            "kubernetes": KubePackageHandler(self.root_dir),
            "middleware": MiddlewarePackageHandler(self.root_dir),
            "harbor": HarborPackageHandler(self.root_dir),
            "license": LicensePackageHandler(self.root_dir),
            "service": service_list
        }
        return to_object(res)


class KubePackageHandler(PackageHandler):
    def __init__(self, path: str):
        super(KubePackageHandler, self).__init__(path, "kubernetes")

    @status_me("basesystem")
    def kubernetes_unpack(self):
        cmd = "tar zxf {} -C {}".format(path_join(self.location, self.package_name), self.location)
        res = run_local_cmd(cmd)
        if res.code == 0 and os.path.isdir(self.root_dir):
            return True
        raise HandlerError(PackageErrors().UNPACK_ERROR.format(res.stdout)) from None

    def parse(self):
        res = {
            "templates": path_join(self.root_dir, "templates"),
            "plugin": {
                "network": path_join(self.root_dir, "plugin/network"),
                "helm": path_join(self.root_dir, "plugin/helm"),
                "istio": path_join(self.root_dir, "plugin/istio"),
                "rook": path_join(self.root_dir, "plugin/rook-ceph"),
                "chaosblade": path_join(self.root_dir, "plugin/chaosblade")
            },
            "k8s-rpms": path_join(self.root_dir, "k8s-rpms")
        }
        return to_object(res)


class MiddlewarePackageHandler(PackageHandler):
    def __init__(self, path: str):
        super(MiddlewarePackageHandler, self).__init__(path, "middleware")

    def parse(self):
        """
        返回中间件列表
        :return: ["mysql", "redis", "minio"]
        """
        middlewares = os.listdir(self.root_dir)
        res = {}
        for m in middlewares:
            res[m] = path_join(self.root_dir, m)
        return to_object(res)


class HarborPackageHandler(PackageHandler):
    def __init__(self, path: str):
        super(HarborPackageHandler, self).__init__(path, "harbor")

    def parse(self):
        res = {
            "harbor": self.root_dir
        }
        return to_object(res)


class LicensePackageHandler(PackageHandler):
    def __init__(self, path: str):
        super(LicensePackageHandler, self).__init__(path, "license")

    def parse(self):
        config_file = ServiceConfigHandler(path_join(self.root_dir, "config.yaml"))
        res = {
            "config": config_file,
            "image": path_join(self.root_dir, "image"),
            "charts": path_join(self.root_dir, "chart"),
            "templates": path_join(self.root_dir, "templates"),
            "sqls": path_join(self.root_dir, "sqls"),
            "siber_sqls": path_join(self.root_dir, "siber_sqls"),
            "identity_sqls": path_join(self.root_dir, "identity_sqls"),
            "etcd_sign": path_join(self.root_dir, "etcd_sign.json"),
            "data": path_join(self.root_dir, "data")
        }
        return to_object(res)


class ServicePackageHandler(PackageHandler):
    def __init__(self, path: str, name: str):
        super(ServicePackageHandler, self).__init__(path, name)

    def parse(self):
        config_file = ServiceConfigHandler(path_join(self.root_dir, "config.yaml"))
        res = {
            "config": config_file,
            "image": path_join(self.root_dir, "image"),
            "charts": path_join(self.root_dir, "chart"),
            "template": path_join(self.root_dir, "template"),
            "sqls": path_join(self.root_dir, "sqls"),
            "siber_sqls": path_join(self.root_dir, "data", "siber_sqls"),
            "siber_sqls_ok": path_join(self.root_dir, "data", "siber_sqls_ok"),
            "identity_sqls": path_join(self.root_dir, "identity_sqls"),
            "data": path_join(self.root_dir, "data"),
            "tf_server_chart": path_join(self.root_dir, "chart"),
            "tf_server_data": path_join(self.root_dir, "data"),
            "test": path_join(self.root_dir, "test")
        }
        return to_object(res)


class OcrPackageHandler(PackageHandler):
    def __init__(self, path: str, name: str):
        super(OcrPackageHandler, self).__init__(path, name)

    def parse(self):
        config_file = ServiceConfigHandler(path_join(self.root_dir, "config.yaml"))
        res = {
            "config": config_file,
            "image": path_join(self.root_dir, "image"),
            "charts": path_join(self.root_dir, "chart"),
            "templates": path_join(self.root_dir, "docker-compose.tmpl"),
            "license_file": path_join(self.root_dir, "data", "licServer.lic")
        }
        return to_object(res)
    
class PreCheckPackageHandler(PackageHandler):
    def __init__(self, path: str, name: str):
        super(PreCheckPackageHandler, self).__init__(path, name)

    def parse(self):
        res = {
            "checker": self.root_dir,
            "check_config": path_join(self.root_dir, "check_config.json")
        }
        return to_object(res)


from __future__ import absolute_import
from __future__ import unicode_literals

import requests
from laipvt.interface.middlewareinterface import MiddlewareInterface
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.handler.middlewarehandler import EtcdConfigHandler
from laipvt.sysutil.util import ssh_obj, log, status_me


class EtcdController(MiddlewareInterface):
	def __init__(self, result: CheckResultHandler, handler: EtcdConfigHandler, template: str):
		super(EtcdController, self).__init__(result, handler, template)
		self.etcd_cfg = EtcdConfigHandler().get_config_with_check_result()
		self.etcd_cfg["etcd"]["ipaddress"] = self.handler.cfg["ipaddress"]

	def _generic_config(self):
		log.info("渲染 Etcd 配置文件")
		if len(self.master_server) == 1:
			log.info("使用单机模式部署")
			self.etcd_cfg["is_standalone"] = True
		elif len(self.master_server) == 3:
			log.info("使用集群模式部署")
			self.etcd_cfg["is_standalone"] = False
		else:
			log.info("集群模式需要3台主机,请检查前置检查配置是否正确")
			exit()
		for num_id in range(len(self.master_server)):
			self.etcd_cfg["etcd"]["etcd_num"] = "etcd%s" % (num_id + 1)
			self.etcd_cfg["etcd"]["ip"] = self.master_server[num_id].ipaddress
			self.generate_docker_compose_file(self.etcd_cfg)
			self._send_docker_compose_file(self.master_server[num_id])


	def _send_docker_compose_file(self, host):
		log.info("分发%s docker-compose配置文件至 %s" % (self.middleware_name, host.ipaddress))
		ssh_cli = ssh_obj(ip=host.ipaddress, user=host.username, password=host.password, port=host.port)
		try:
			ssh_cli.put(self.docker_compose_file_tmp, self.docker_compose_file)
		except Exception as e:
			log.error(e)
			exit(2)
		finally:
			ssh_cli.close()

	def _check(self):
		super().wait_for_service_start()
		for i in range(len(self.master_server)):
			try:
				log.info("检查 {IP}:{PORT} 上的 Etcd 健康性...".format(
					IP=self.master_server[i].ipaddress, PORT=self.etcd_cfg["etcd"]["http_port"]))
				requests.get(
					"http://{IP}:{PORT}/version".format(
						IP=self.master_server[i].ipaddress, PORT=self.etcd_cfg["etcd"]["http_port"]
					)
				)
				log.info("Etcd检查通过")
			except Exception as e:
				log.error(e)
				log.error("{IP}:{PORT} 上的 Etcd 服务异常".format(
					IP=self.master_server[i].ipaddress), PORT=self.etcd_cfg["etcd"]["http_port"])
				exit(2)

	@status_me("middleware")
	def deploy_etcd(self):
		self._generic_config()
		self.start()
		self._check()

	def deploy(self):
		self.deploy_etcd()



from __future__ import absolute_import
from __future__ import unicode_literals

import os
import time

import requests
from minio import Minio
from hashlib import sha256

from laipvt.interface.serviceinterface import ServiceInterface
from laipvt.helper.errors import Helper
from laipvt.sysutil.command import COMMANDER_UPGRADE_INSTALL, CREATE_DB, HELM_INSTALL_COMMANDER, HELM_LIST
from laipvt.sysutil.util import log, path_join, status_me, walk_sql_path, CommanderAes


class CommanderController(ServiceInterface):
    def __init__(self, check_result, service_path):
        super(CommanderController, self).__init__(check_result, service_path)

        self.nginx_template = path_join(self.templates_dir, "nginx/nginx-entcmd.tmpl")
        self.nginx_tmp = path_join("/tmp", "nginx-entcmd.conf")
        self.nginx_file_remote = path_join(self.deploy_dir, "nginx/http/nginx-entcmd.conf")
        if self.middleware_cfg["mysql"]["is_deploy"]:
            self.mysql_host = "mysql.default.svc"
            self.mysql_port = 3306
        else:
            self.mysql_host = self.middleware_cfg["mysql"]["ipaddress"][0]
            self.mysql_port = self.middleware_cfg["mysql"]["port"]

        # self.protocol = "https" if self.middleware_cfg["config"]["deploy_https"] else "http"
        # self.tenant_url = "{protocol}://{host}:{port}".format(protocol=self.protocol,
        #                                                       host=self.middleware_cfg.nginx.lb,
        #                                                       port=self.middleware_cfg.nginx.commander_tenant_port)

    @status_me("commander")
    def deploy_configmap(self):
        self.deploy_all_configmap()

    @status_me("commander")
    def deploy_commander_istio(self):
        self.deploy_istio()

    @status_me("commander")
    def init_commander_mysql(self):
        self.init_mysql()

    @status_me("commander")
    def init_commander_rabbitmq(self):
        self.init_rabbitmq()

    @status_me("commander")
    def init_commander_redis(self):
        self.init_redis()

    @status_me("commander")
    def init_commander_minio(self):
        self.init_minio()

    @status_me("commander")
    def push_commander_images(self):
        self.push_images(self.project)

    def upgrade_service(self, project):
        for service, processes in self.service_path.config.services.items():
            for process in processes:
                log.info("{}开始更新".format(process))
                self._create_logs_dir(service)
                file_path = os.path.join(self.service_path.charts, process)

                if self.middleware_cfg["redis"]["is_deploy"]:
                    config_server = "\,".join(
                            [
                                    "{}:{}\,serviceName={}\,allowAdmin=true".format(
                                            server,
                                            self.middleware_cfg["redis"]["port_sentinel"],
                                            self.middleware_cfg["redis"]["master_name"]
                                    ) for server in self.middleware_servers.get_role_ip("master")
                            ]
                    )
                else:
                    config_server = "{}:{}".format(self.middleware_cfg["redis"]["ipaddress"][0],
                                                   self.middleware_cfg["redis"]["port"])

                cmd = COMMANDER_UPGRADE_INSTALL.format(
                        process=process, replicas=self.replicas,
                        registry_hub=path_join(self.registry_hub, project),
                        image_name=process, image_tag=self.private_deploy_version,
                        pvt_work_dir=self.deploy_dir,
                        config_server=config_server,
                        config_server_passwd=self.middleware_cfg["redis"]["password"],
                        mysql_host=self.mysql_host, mysql_port=self.mysql_port,
                        mysql_user=self.middleware_cfg["mysql"]["username"],
                        mysql_password=self.middleware_cfg["mysql"]["password"],
                        etcd_endpoint=self.etcd_endpoint, mysql_database="uibot_global",
                        mysql_charset="utf8mb4",
                        oidc_authority="http://{}:{}".format(
                                self.middleware_cfg["identity"]["lb"],
                                self.middleware_cfg["identity"]["nginx_proxy_port"]),
                        oidc_secret="laiye",
                        file_path=file_path)

                self._exec_command_to_host(cmd=cmd, server=self.servers[0])

    @status_me("commander")
    def start_commander_service(self):
        self.start_service(project=self.project, version=self.private_deploy_version)

    @status_me("commander")
    def commander_proxy_on_nginx(self):
        self.proxy_on_nginx(self.nginx_template, self.nginx_tmp, self.nginx_file_remote)

    def login_tenant(self):
        log.info(Helper().LOGIN_TENANT)
        login_account_url = "{tenant_url}/api/tenant/account/getCurrentInfo".format(tenant_url=self.tenant_url)
        AccountResponse = requests.get(login_account_url)  # goto get the xsrf-token for post request.
        preCookies = AccountResponse.cookies  # include xsrf-token for future use.
        XsrfTOKEN = preCookies.get('XSRF-TOKEN')  # need this to header.
        loginHeaders = {
                'X-XSRF-TOKEN': XsrfTOKEN,
                'Referer':      self.tenant_url
        }
        # 将默认密码加密：先将token使用sha256加密，然后取其前32位，再使用aes加密
        tenant_password = "123456".ljust(16, "\n")
        sha256_token = sha256(XsrfTOKEN.encode('utf-8')).hexdigest()[:32]
        password = CommanderAes.encrypt_oracle(sha256_token, tenant_password)
        data = {'userName': 'admin', 'password': password}
        login_url = "{tenant_url}/api/global/account/webLogin".format(tenant_url=self.tenant_url)
        response = requests.post(login_url, json=data, headers=loginHeaders, cookies=preCookies,
                                 verify=False)
        auth_cookie = ''
        # print(response)
        ret = response.json()
        if ret['code'] == 0:
            auth_cookies = preCookies
            auth_cookies.set('GlobalUser', response.cookies.get('GlobalUser'))
            # f"GlobalUser={cookies.get('GlobalUser', None)}"
            # log.info(auth_cookies)
            log.info("Login succeed")
            return auth_cookies, loginHeaders
        else:
            log.error("Login error!")
            exit(2)

    def tenant_init_mysql(self, auth_cookies, login_headers):
        log.info("租户平台配置MySql数据库")
        db_mysql = {
                "name":     "mysql-1",
                "host":     self.mysql_host,
                "port":     self.mysql_port,
                "dbName":   "uibot_rpa",
                "userName": self.middleware_cfg.mysql.username,
                "password": self.middleware_cfg.mysql.password,
                "type":     "10"
        }
        mysql_url = "{tenant_url}/api/global/database/create".format(tenant_url=self.tenant_url)
        resp1 = requests.post(mysql_url, json=db_mysql, cookies=auth_cookies, headers=login_headers, verify=False)
        json_resp1 = resp1.json()
        if json_resp1['code'] == 0:
            log.info("配置MySQL数据库完成")
        else:
            log.error("配置MySQL数据库失败")
            exit(2)

    @status_me("commander")
    def init_tenant(self):
        counter = 0
        succeed = False
        # 重试 10 次，如果还是不成功就报错
        while not succeed and counter < 100:
            time.sleep(5)
            try:
                auth_cookies, loginHeaders = self.login_tenant()
                self.tenant_init_mysql(auth_cookies=auth_cookies, login_headers=loginHeaders)
                succeed = True
            except Exception as e:
                # log.error(e)
                succeed = False
                counter += 1
        if not succeed:
            log.error("登录租户系统失败，请检查容器是否启动。")
            exit(2)

    @status_me("commander")
    def commander_apptest(self):
        self.app_test(project=self.project)

    @status_me("commander")
    def init_commander_entuc_clients(self):
        self.init_usercenter_clients(project=self.project)

    def run_apptest(self):
        self.app_test(project=self.project)

    def run(self):
        self.init_commander_mysql()
        self.init_commander_rabbitmq()
        self.init_commander_minio()
        self.deploy_configmap()
        self.push_commander_images()
        self.deploy_commander_istio()
        self.start_commander_service()
        # self.commander_proxy_on_nginx()
        self.project_pod_check()
        # self.init_commander_entuc_clients()
        self.commander_apptest()



from __future__ import absolute_import
from __future__ import unicode_literals

from laipvt.sysutil.ssh import SSHConnect
from laipvt.interface.serviceinterface import ServiceInterface
from laipvt.sysutil.util import path_join, log, status_me


def ssh_obj(ip, user, password, port=22) -> SSHConnect:
    return SSHConnect(hostip=ip, username=user, password=password, port=port)


class ChatbotController(ServiceInterface):
    def __init__(self, check_result, service_path):
        super(ChatbotController, self).__init__(check_result, service_path)

        self.chatbot_app_config_hosts = path_join(self.config , "Chatbot/laiye-chatbot-app.conf")
        self.chatbot_app_config_container = "/home/works/program/conf/online.conf"

        self.nginx_template = path_join(self.templates_dir, "nginx/nginx-chatbot.tmpl")
        self.nginx_tmp = path_join("/tmp", "nginx-chatbot.conf")
        self.nginx_file_remote = path_join(self.deploy_dir, "nginx/http/nginx-chatbot.conf")

    @status_me("chatbot")
    def deploy_chatbot_configmap(self):
        self.deploy_all_configmap()

    @status_me("chatbot")
    def deploy_chatbot_istio(self):
        self.deploy_istio()

    @status_me("chatbot")
    def init_chatbot_mysql(self):
        log.info("初始化mysql数据")
        # version = self.private_deploy_version.split("-")[0]
        mysql_real_ip = self.master_host.ipaddress
        if not self.middleware_cfg.mysql.is_deploy:
            mysql_real_ip = self.middleware_cfg.mysql.ipaddress[0]

        # docker run -t {进程：chatbot-app} --init
        cmd = "docker run --add-host mysql.default.svc:{} -t -v {}:{} {}/{}/chatbot-app:{} --init".format(
            mysql_real_ip,
            self.chatbot_app_config_hosts, self.chatbot_app_config_container,
            self.registry_hub, self.project, self.private_deploy_version
        )
        res = self._exec_command_to_host(cmd=cmd, server=self.servers[0], check_res=True)
        if res["code"] != 0:
            log.error("初始化mysql数据失败")
            log.error(res["stdout"])
            exit(2)
        log.info("初始化mysql数据完成")

    @status_me("chatbot")
    def push_chatbot_images(self):
        self.push_images(self.project)

    @status_me("chatbot")
    def start_chatbot_service(self):
        self.start_service(project=self.project, version=self.private_deploy_version)

    @status_me("chatbot")
    def chatbot_proxy_on_nginx(self):
        self.proxy_on_nginx(self.nginx_template, self.nginx_tmp, self.nginx_file_remote)

    @status_me("chatbot")
    def prepare_data_chatbot(self):
        self.prepare_data(project=self.project)

    @status_me("chatbot")
    def chatbot_apptest(self):
        self.app_test(project=self.project)

    @status_me("chatbot")
    def init_chatbot_entuc_clients(self):
        self.init_usercenter_clients(project=self.project)

    def run_apptest(self):
        self.app_test(project=self.project)

    def run(self):
        self.push_chatbot_images()
        self.deploy_chatbot_configmap()
        self.deploy_chatbot_istio()
        self.init_chatbot_mysql()
        self.start_chatbot_service()
        self.chatbot_proxy_on_nginx()
        self.prepare_data_chatbot()
        self.project_pod_check()
        self.init_chatbot_entuc_clients()
        self.chatbot_apptest()



from __future__ import absolute_import
from __future__ import unicode_literals

import socket
import re
import os

def is_port(port):
    try:
        return True if int(port) >= 0 and int(port) < 65536 else False
    except Exception:
        return False

def is_ip_or_fqdn(ip):
    try:
        res = socket.getaddrinfo(ip, 0, socket.AF_UNSPEC,
                                 socket.SOCK_STREAM,
                                 0, socket.AI_NUMERICHOST)
        return bool(res)
    except Exception:
        pattern = re.compile(
            r'^(([a-zA-Z]{1})|([a-zA-Z]{1}[a-zA-Z]{1})|'
            r'([a-zA-Z]{1}[0-9]{1})|([0-9]{1}[a-zA-Z]{1})|'
            r'([a-zA-Z0-9][-_.a-zA-Z0-9]{0,61}[a-zA-Z0-9]))\.'
            r'([a-zA-Z]{2,13}|[a-zA-Z0-9-]{2,30}.[a-zA-Z]{2,3})$'
        )
        return True if pattern.match(ip) else False

def is_password(password: str) -> bool:
    try:
        return True if re.search("^(?=.*\d)(?=.*[a-z])(?=.*[A-Z]).*$", password) and len(password) >= 8 else False
    except Exception:
        return False

def is_dir(path: str) -> bool:
    return os.path.isdir(path)

def is_file(path: str) -> bool:
    return os.path.isfile(path)


# Char
from locale import getdefaultlocale


def getChar():
    char = getdefaultlocale()[0].lower()
    if "zh" in char:
        DEPLOY_LANGUAGE = "cn"
    else:
        DEPLOY_LANGUAGE = "en"
    return DEPLOY_LANGUAGE


CHECK_INTERVAL = 20
DEPLOY_LANGUAGE = getChar()
LAIPVT_BASE_DIR = "/usr/local/laipvt"
CHECK_FILE = "/usr/local/laipvt/check_result.yaml"
PRE_CHECK_RESULT_FILE = "/usr/local/laipvt/.precheck_result.yaml"
PORT_MAP = "/usr/local/laipvt/ports.yaml"
STATUS_FILE = "/usr/local/laipvt/.deploy_status"
PROJECT_INFO_FILE = "/usr/local/laipvt/project"
UPGRADE_INFO_FILE = "/usr/local/laipvt/upgrade"
ENTUC_JSON_FILE = "/usr/local/laipvt/entuc.json"

# LOG
LAIPVT_LOG_PATH = "/usr/local/laipvt/logs"
LAIPVT_LOG_NAME = "laipvt.log"
LAIPVT_LOG_LEVEL = "DEBUG"
LOG_TO_TTY = True

# Certs
CERTS_PATH = "/usr/local/laipvt/certs"

# Precheck temp
CHECK_TEMP = "/tmp"



from __future__ import absolute_import
from __future__ import unicode_literals
import os, json
from laipvt.interface.serviceinterface import ServiceInterface
from laipvt.handler.middlewarehandler import EtcdConfigHandler
from laipvt.sysutil.util import path_join, log, status_me
from laipvt.controller.middleware.etcd import EtcdController
from laipvt.helper.errors import Helper
from laipvt.sysutil.command import GET_POD_INFO, RESTART_DEPLOYMENT, RESTART_DEPLOYMENT_ALL, LICENSE_QRCODE_CMD
from laipvt.model.cmd import DockerModel, DockerImageModel
from laipvt.model.server import runcmd


class LicenseController(ServiceInterface):
    def __init__(self, check_result, service_path, imagePkgPath=""):
        super(LicenseController, self).__init__(check_result, service_path)
        self.nginx_template = path_join(self.templates_dir, "nginx/nginx-license.tmpl")
        self.nginx_tmp = path_join("/tmp", "nginx-license.conf")
        self.nginx_file_remote = path_join(self.deploy_dir, "nginx/http/nginx-license.conf")
        self.imagePkgPath = imagePkgPath

    @status_me("license")
    def prepare_license_data_file(self, is_renew=False, **kwargs):
        # if is_renew:
        #     lcs_file_path = kwargs.get("license_file")
        # else:
        #     # 找到以.lcs结尾的文件
        #     lcs_file_path_list = list(filter(lambda x: x.endswith(".lcs"), os.listdir(self.service_path.data)))
        #     if lcs_file_path_list:
        #         lcs_file_path = path_join(self.service_path.data, lcs_file_path_list[0])
        #
        #     else:
        #         log.error(Helper().FILE_NOT_FOUND.format("license file"))
        #         exit(2)
        self.prepare_data(project=self.project)

    def restart_service(self):
        etcd = EtcdController(self.check_result, EtcdConfigHandler(), "")
        etcd.reset()
        get_pod_info_cmd = GET_POD_INFO.format("license-manager")
        res = self._exec_command_to_host(cmd=get_pod_info_cmd, server=self.harbor_hosts[0], check_res=True)
        if not (res["code"] == 0 and "Running" in res["stdout"]):
            log.error("license service not found:{}".format(get_pod_info_cmd))
            exit(2)
        restart_deployment_cmd =  RESTART_DEPLOYMENT.format("license", "license-manager")
        self._exec_command_to_host(cmd=restart_deployment_cmd, server=self.harbor_hosts[0], check_res=True)

        restart_all_deployment_cmd = [
            RESTART_DEPLOYMENT_ALL.format("mage"),
            RESTART_DEPLOYMENT_ALL.format("rpa")
        ]
        self._exec_command_to_host(cmd=restart_all_deployment_cmd, server=self.harbor_hosts[0], check_res=True)

    def renew_license(self, license_file):
        self.prepare_license_data_file(is_renew=True, license_file=license_file)
        self.restart_service()

    def runqrcode(self, path):
        global image_name_withtag
        image_name = path_join(path, "license_feature.tar")
        try:
            image_name_withtag = DockerModel().load(image_name)
        except Exception as e:
            log.error(e)
        log.info(image_name)
        srcConfig = path_join(self.deploy_dir, "license_config/License/laiye-saas-license-manager.conf")
        destConfig = "/home/works/program/conf/conf.yaml"
        cmd = LICENSE_QRCODE_CMD.format(
            self.harbor_hosts[0].ipaddress,
            srcConfig, destConfig, path, path, image_name_withtag[0], path
        )
        # self._exec_command_to_host(cmd, server=self.harbor_hosts[0], check_res=True)
        log.info(cmd)
        code, res = runcmd(cmd)
        if code != 0:
            log.error(res)
            exit(2)
        headStr = res.stdout.find("QR Code")
        tailStr = res.stdout.find(".jpg") + 5
        if res.stdout[headStr:tailStr] == "":
            log.debug("未获取到QR CODE字段，请手动执行生成特征码命令查看具体原因")
            log.info(cmd)
        else:
            log.info("请将服务器中特征码照片文件下载到本地并上传至来也内部私有发货平台进行授权文件申请，特征码文件存放位置: {}".format(res.stdout[headStr:tailStr]))

    @status_me("license")
    def push_license_images(self):
        self.push_images(self.project)

    @status_me("license")
    def init_license_mysql(self):
        self.init_mysql()

    @status_me("license")
    def start_license_service(self):
        self._create_namespace(namespaces=self.namespaces, istio_injection_namespaces="")
        self.start_service(project=self.project, version=self.private_deploy_version)

    @status_me("license")
    def license_proxy_on_nginx(self):
        self.proxy_on_nginx(self.nginx_template, self.nginx_tmp, self.nginx_file_remote)

    @status_me("license")
    def deploy_license_configmap(self):
        self.deploy_all_configmap()

    @status_me("license")
    def deploy_license_istio(self):
        self.deploy_istio()

    @status_me("license")
    def init_license_entuc_clients(self):
        self.init_usercenter_clients(project="license")

    def run(self):
        self.push_license_images()
        self.deploy_license_configmap()
        self.init_license_mysql()
        self.deploy_license_istio()
        self.runqrcode(self.imagePkgPath)
        self.start_license_service()
        self.prepare_license_data_file()
        # self.license_proxy_on_nginx()
        self.project_pod_check()
        # self.init_license_entuc_clients()

    def runQrcode(self):
        self.runqrcode(self.imagePkgPath)



from __future__ import absolute_import
from __future__ import unicode_literals

from laipvt.interface.serviceinterface import ServiceInterface
from laipvt.sysutil.util import path_join, log, status_me


class NoticeController(ServiceInterface):
    def __init__(self, check_result, service_path):
        super(NoticeController, self).__init__(check_result, service_path)
        self.nginx_template = path_join(self.templates_dir, "nginx/nginx-notice.tmpl")
        self.nginx_tmp = path_join("/tmp", "nginx-notice.conf")
        self.nginx_file_remote = path_join(self.deploy_dir, "nginx/http/nginx-notice.conf")


    @status_me("notice")
    def push_notice_images(self):
        self.push_images(self.project)

    @status_me("notice")
    def deploy_notice_configmap(self):
        self.deploy_all_configmap()

    @status_me("notice")
    def deploy_notice_istio(self):
        self.deploy_istio()

    @status_me("notice")
    def start_notice_service(self):
        self._create_namespace(namespaces=self.namespaces,
                               istio_injection_namespaces=self.istio_injection_namespaces)
        self.start_service(project=self.project, version=self.private_deploy_version)

    @status_me("notice")
    def prepare_notice_data(self):
        self.prepare_data(project=self.project)

    @status_me("notice")
    def notice_proxy_on_nginx(self):
        self.proxy_on_nginx(self.nginx_template, self.nginx_tmp, self.nginx_file_remote)

    @status_me("notice")
    def notice_apptest(self):
        self.app_test(project=self.project)

    @status_me("notice")
    def init_notice_mysql(self):
        self.init_mysql()

    def run_apptest(self):
        self.app_test(project=self.project)

    def run(self):
        self.push_notice_images()
        self.init_notice_mysql()
        self.deploy_notice_istio()
        self.deploy_notice_configmap()
        self.start_notice_service()
        # self.notice_proxy_on_nginx()
        self.prepare_notice_data()
        self.project_pod_check()
        self.notice_apptest()




# -*- encoding: utf-8 -*-
import argparse


def Args():
    parser = argparse.ArgumentParser(description='Process some integers.')
    parser.add_argument('--force', dest="isCheck", action="store_true", default=False, help="前置检查效验")
    parser.add_argument('--sikp-kubernetes', dest="sikpKubernetes", action="store_true", default=False, help="是否跳过kubernetes部署")
    parser.add_argument('-f', '--targz-file', dest="targzFile", type=str, help="指定部署压缩包")
    parser.set_defaults(which='')

    subparsers = parser.add_subparsers(title='laipvt', description='命令模块分组', help='命令模块分组')

    deploy_parser = subparsers.add_parser('license', help='授权功能相关参数')
    deploy_parser.add_argument('--license-file', dest="LicenseFile",
                               type=str, help="指定需要更新的授权文件")
    deploy_parser.add_argument('--ocr-license-file', dest="OcrLicenseFile",
                               type=str, help="指定需要更新的ocr授权文件")
    deploy_parser.set_defaults(which='license')


    add_parser = subparsers.add_parser('deploy', help='部署额外功能相关参数')
    add_parser.add_argument('--monitor', dest="Monitor", action="store_true",
                                default=False, help="部署监控功能")
    add_parser.add_argument('--keepalive', dest="Keepalive", action="store_true",
                                default=False, help="部署keepalive服务")
    add_parser.set_defaults(which='add')


    delete_parser = subparsers.add_parser('delete', help='删除功能相关参数')
    delete_parser.add_argument("--docker", dest="Docker", action="store_true",
                                default=False, help="删除docker服务")
    delete_parser.add_argument('--kubernetes', dest="Kubernetes", action="store_true",
                                default=False, help="删除kubernetes服务")
    delete_parser.add_argument('--middleware', dest="Middleware", action="store_true",
                                default=False, help="删除middleware中间件服务")
    delete_parser.add_argument('--all', dest="All", action="store_true",
                                default=False, help="删除所有服务")
    delete_parser.set_defaults(which='delete')

    reset_parser = subparsers.add_parser('resetdata', help='重置中间件数据')
    reset_parser.add_argument("--service", dest="Service", type=str, help="指定需要自动化测试的项目")
    reset_parser.add_argument('-f', '--targz-file', dest="targzFile", type=str, help="指定部署压缩包")
    reset_parser.set_defaults(which='')

    apptest_parser = subparsers.add_parser('apptest', help='自动化功能相关参数')
    apptest_parser.add_argument("--service", dest="Service", type=str, help="指定需要自动化测试的项目")
    apptest_parser.set_defaults(which='test')

    return parser


if __name__ == '__main__':
    args = Args().parse_args()
    print(args)



#!/bin/env python
# -*- encoding: utf-8 -*-
import json
import os
from laipvt.sysutil.conf import status_file


dict_tmpl = {
    "basesystem": {
        "kubernetes_unpack": 0,
        "install_harbor": 0,
        "install_nginx": 0,
        "add_hosts": 0,
        "system_prepare": 0,
        "init_primary_master": 0,
        "cp_kube_config": 0,
        "kube_completion": 0,
        "install_network_plugin": 0,
        "join_master": 0,
        "join_node": 0,
        "install_helm": 0,
        "install_istio": 0,
        "install_rook": 0,
        "install_nvidia_device_plugin": 0,
        "install_chaosblade": 0,
        "create_namespace": 0
    },
    "nginx": {
        "renew_apiserver_config": 0
    },
    "middleware": {
        "deploy_etcd": 0,
        "deploy_license": 0,
        "deploy_minio": 0,
        "deploy_mysql": 0,
        "deploy_redis": 0,
        "deploy_es": 0,
        "deploy_rabbitmq": 0,
        "deploy_identity": 0,
        "deploy_commander_identity": 0,
        "deploy_monitor": 0,
        "deploy_keepalived": 0,
        "deploy_siber": 0
    },
    "license": {
        "push_license_images": 0,
        "deploy_license_configmap": 0,
        "deploy_license_istio": 0,
        "prepare_license_data_file": 0,
        "start_license_service": 0,
        "license_proxy_on_nginx": 0,
        "init_license_entuc_clients": 0
    },
    "mage": {
        "init_mage_mysql": 0,
        "init_identity_user": 0,
        "init_minio_data": 0,
        "push_mage_images": 0,
        "deploy_configmap": 0,
        "deploy_mage_istio": 0,
        "start_mage_service": 0,
        "mage_proxy_on_nginx": 0,
        "mage_transfer_data": 0,
        "hide_menu": 0,
        "init_mage_entuc_clients": 0,
        "apply_mage_model_svc": 0
    },
    "chatbot": {
        "push_chatbot_images": 0,
        "init_chatbot_mysql": 0,
        "deploy_chatbot_configmap": 0,
        "deploy_chatbot_istio": 0,
        "start_chatbot_service": 0,
        "chatbot_proxy_on_nginx": 0,
        "prepare_data_chatbot": 0,
        "chatbot_apptest": 0,
        "init_chatbot_entuc_clients": 0
    },
    "nlp": {
        "push_nlp_images": 0,
        "deploy_nlp_configmap": 0,
        "start_nlp_service": 0,
        "prepare_nlp_data": 0
    },
    "ocr": {
        "push_ocr_images": 0,
        "deploy_ocr_configmap": 0,
        "start_ocr_service": 0,
        "prepare_ocr_data": 0,
        "apply_ocr_model_svc": 0
    },
    "ocr_text": {
        "push_text_images": 0,
        "start_text_service": 0,
        "prepare_text_data": 0
    },
    "ocr_text_cpu": {
        "push_text_images": 0,
        "start_text_service": 0,
        "prepare_text_data": 0
    },
    "ocr_table": {
        "push_table_images": 0,
        "start_table_service": 0,
        "prepare_table_data": 0
    },
    "ocr_bizlicense": {
        "push_bizlicense_images": 0,
        "start_bizlicense_service": 0,
        "prepare_bizlicense_data": 0
    },
    "nlp_layoutlm": {
        "push_nlp_layoutlm_images": 0,
        "start_nlp_layoutlm_service": 0,
        "prepare_nlp_layoutlm_data": 0
    },
    "ocr_idcard": {
        "push_ocr_idcard_images": 0,
        "start_ocr_idcard_service": 0,
        "prepare_ocr_idcard_data": 0
    },
    "captcha": {
        "push_captcha_images": 0,
        "start_captcha_service": 0,
        "prepare_captcha_data": 0
    },
    "seal": {
        "push_seal_images": 0,
        "start_seal_service": 0,
        "prepare_seal_data": 0
    },
    "ocr_layout_analysis": {
        "push_layout_analysis_images": 0,
        "start_layout_analysis_service": 0,
        "prepare_layout_analysis_data": 0
    },
    "rpa_collaboration": {
        "push_rpa_collaboration_images": 0,
        "init_rpa_collaboration_mysql": 0,
        "init_rpa_collaboration_minio": 0,
        "deploy_rpa_collaboration_configmap": 0,
        "deploy_rpa_collaboration_istio": 0,
        "start_rpa_collaboration_service": 0,
        "prepare_rpa_collaboration_data": 0,
        "rpa_collaboration_proxy_on_nginx": 0,
        "init_rpa_collaboration_entuc_clients": 0,
        "rpa_collaboration_apptest": 0
    },
    "commander": {
        "init_commander_mysql": 0,
        "init_commander_rabbitmq": 0,
        "init_minio_data": 0,
        "init_commander_redis": 0,
        "deploy_configmap": 0,
        "deploy_commander_istio": 0,
        "push_commander_images": 0,
        "start_commander_service": 0,
        "commander_proxy_on_nginx": 0,
        "init_tenant": 0,
        "commander_apptest": 0,
        "init_commander_entuc_clients": 0
    },
    "usercenter": {
        "push_usercenter_images": 0,
        "init_minio_data": 0,
        "init_usercenter_mysql": 0,
        "init_usercenter_rabbitmq": 0,
        "deploy_configmap": 0,
        "deploy_usercenter_istio": 0,
        "start_usercenter_service": 0,
        "usercenter_proxy_on_nginx": 0,
        "usercenter_init_user": 0
    },
    "dataservice": {
        "push_dataservice_images": 0,
        "deploy_dataservice_configmap": 0,
        "start_dataservice_service": 0,
        "dataservice_proxy_on_nginx": 0,
        "dataservice_apptest": 0,
        "deploy_dataservice_istio": 0,
        "prepare_dataservice_data": 0
    },
    "notice": {
        "push_notice_images": 0,
        "init_notice_mysql": 0,
        "deploy_notice_istio": 0,
        "deploy_notice_configmap": 0,
        "start_notice_service": 0,
        "notice_proxy_on_nginx": 0,
        "prepare_notice_data": 0,
        "notice_apptest": 0
    }
}


class Status:
    def __init__(self):
        self.status_file = status_file
        self.STATUS_SUCCESS = 1
        self.STATUS_FAILED = 2
        self.STATUS_NOT_RUNNING = 0
        self.status_dicts = [self.STATUS_NOT_RUNNING, self.STATUS_SUCCESS, self.STATUS_FAILED]
        if os.path.exists(self.status_file):
            with open(self.status_file, 'r') as fp:
                self.status = json.load(fp)
        else:
            with open(self.status_file, 'a') as fp:
                fp.write(json.dumps(dict_tmpl, indent=4))
            self.status = dict_tmpl

    def reset_status(self):
        status = json.loads(json.dumps(dict_tmpl))
        with open(self.status_file, "w") as sf:
            json.dump(status, sf, indent=4)

    def _reload(self):
        with open(self.status_file) as sf:
            self.status = json.load(sf)

    def _update(self):
        with open(self.status_file, "w") as sf:
            json.dump(self.status, sf, indent=4)
        self._reload()

    def get_status_failed(self, project_name):
        step_list = []
        proj = self.status[project_name]
        for step in proj:
            if proj[step] == self.STATUS_FAILED:
                step_list.append(step)
        return step_list

    def get_status(self, project, step):
        try:
            self._reload()
            return self.status_dicts[self.status[project][step]]
        except KeyError:
            self.update_status(project, step, 0)
            self._reload()
            return self.status_dicts[self.status[project][step]]

    def update_status(self, project, step, value):
        try:
            self.status[project][step] = int(self.status_dicts[value])
            self._update()
            return True
        except IndexError:
            return False



from __future__ import absolute_import
from __future__ import unicode_literals

import logging
import os
from logging import handlers
from logging.handlers import RotatingFileHandler as RFHandler


class Logger():
    """
        日志记录类
    """

    # 日志级别关系映射
    level_relations = {
        'debug': logging.DEBUG,
        'info': logging.INFO,
        'warning': logging.WARNING,
        'error': logging.ERROR,
        'crit': logging.CRITICAL
    }

    log_fmt = '[%(levelname)s] %(asctime)s %(filename)s [line:%(lineno)d] %(funcName)s: %(message)s'

    def __init__(self, log_path="/var/log/laipvt", log_name="laipvt.log", log_level="debug", fmt=log_fmt, back_count=3, tty=False):

        if not os.path.exists(log_path):
            os.makedirs(log_path)

        file_name = os.path.join(log_path, log_name)
        format_str = logging.Formatter(fmt)

        self.logger = logging.getLogger(file_name)
        self.logger.setLevel(self.level_relations.get(log_level.lower()))
        file_handler = handlers.TimedRotatingFileHandler(filename=file_name,
                                               when="D",
                                               backupCount=back_count,
                                               encoding='utf-8')
        file_handler.setFormatter(format_str)
        self.logger.addHandler(file_handler)
        if tty:
            stream_handler = logging.StreamHandler()
            stream_handler.setFormatter(format_str)
            self.logger.addHandler(stream_handler)

    def get(self):
        return self.logger


from __future__ import absolute_import
from __future__ import unicode_literals
import subprocess
from laipvt.sysutil.util import ssh_obj, log
from laipvt.helper.errors import RuntimeErrors

class ServerModel():
    def __init__(self, server):
        self.server_obj = {}
        if type(server) == list:
            for s in server:
                self.server_obj[s.ipaddress] = ssh_obj(s.ipaddress, s.username, s.password, port=s.port)
        else:
            self.server_obj[server.ipaddress] = ssh_obj(server.ipaddress, server.username, server.password, port=server.port)

    def _send(self, cli, src, dest, ipaddr):
        try:
            cli.put(src, dest)
        except Exception as e:
            log.error(RuntimeErrors().SEND_FILE_ERROR.format(src, ipaddr, e))

    def _run(self, cli, cmd, ipaddr):
        try:
            cli.run_cmd(cmd)
        except Exception as e:
            log.error(RuntimeErrors().RUN_CMD_ERROR.format(ipaddr, cmd, e))


    def send_file(self, src, dest, ip=""):
        if ip:
            cli = self.server_obj[ip]
            self._send(cli, src, dest, ip)
        else:
            for ip in self.server_obj:
                self._send(self.server_obj[ip], src, dest, ip)

    def exec_cmd(self, cmd, ip=""):
        if ip:
            cli = self.server_obj[ip]
            self._run(cli, cmd, ip)
        else:
            for ip in self.server_obj:
                self._run(self.server_obj[ip], cmd, ip)

    def send_and_exec(self, src, dest, cmd, ip=""):
        if ip:
            cli = self.server_obj[ip]
            self._send(cli, src, dest, ip)
            self._run(cli, cmd, ip)
        else:
            for ip in self.server_obj:
                self._send(self.server_obj[ip], src, dest, ip)
                self._run(self.server_obj[ip], cmd, ip)

    def close(self):
        for ip in self.server_obj:
            self.server_obj[ip].close()

def runcmd(command, timeout=1000):
    ret = subprocess.run(
        command, shell=True,
        stdout=subprocess.PIPE, stderr=subprocess.PIPE,
        encoding="utf-8", timeout=timeout
    )
    return ret.returncode, ret



from laipvt.model.cmd import KubeModel
from laipvt.sysutil.util import log


def wait_pod_running():
    import time
    counter = 0
    succeed = False
    # 每10秒重试一次，重试 10 次，如果还是不成功就报错
    while not succeed and counter < 10:
        time.sleep(10)
        try:
            kube = KubeModel()
            pod_info = kube.get_all_pod_status()
            for pod in pod_info:
                if pod["status"] != "Running":
                    succeed = False
                    counter += 1
                else:
                    break
        except Exception as e:
            log.error(e)
            return False
        if not succeed:
            # log.error("kubernetes集群中有pod启动状态异常，请检查: kubectl get pod -A")
            return False
        else:
            return True



from __future__ import absolute_import
from __future__ import unicode_literals
import os
import json
import time
import requests
from requests.adapters import HTTPAdapter
from minio import Minio
from laipvt.model.cmd import DockerImageModel
from laipvt.model.harbor import HarborModel
from laipvt.helper.exception import UtilsError
from laipvt.handler.confighandler import CheckResultHandler, ServerHandler, PvtAdminConfigHandler
from laipvt.handler.middlewarehandler import HarborConfigHandler
from laipvt.sysutil.template import FileTemplate
from laipvt.sysutil.util import path_join, log, ssh_obj, to_object, walk_sql_path
from laipvt.handler.middlewarehandler import MiddlewareConfigHandler
from laipvt.model.cmd import ComposeModel, KubeModel
from laipvt.model.sql import SqlModule
from laipvt.model.server import runcmd
from laipvt.helper.errors import Helper
from laipvt.sysutil.command import CREATE_NS, ISTIO_INJECTION_NS, KUBECTL_APPLY, MKDIR_DIR, CHMOD_777, \
    HELM_LIST, HELM_INSTALL_CPU, HELM_INSTALL_GPU, CREATE_DB, INIT_IDENTITY_USER, HELM_INSTALL_TF_SERVICE, \
    HELM_UPGRADE_INSTALL, GET_PVC_VOLUMENAME_CMD, RESTART_DEPLOYMENT, TEST_SERVICE_CMD, GET_NFS_NODE_IP_CMD


class ServiceInterface:
    def __init__(self, check_result: CheckResultHandler, service_path):
        """
        check_result: 对象，前置检查结果
        service_path: 对象，服务进程详情
        """
        self.check_result = check_result
        self.service_path = service_path

        self.admin_config_dir = os.path.dirname(os.path.dirname(self.service_path.template))
        self.admin_config_name = list(filter(lambda x: x.endswith("config.json"), os.listdir(self.admin_config_dir)))
        self.admin_config = PvtAdminConfigHandler(path_join(self.admin_config_dir, self.admin_config_name[0]))

        self.middleware_servers = self.check_result.servers
        self.middleware_server_list = self.middleware_servers.get_role_ip("master")

        self.middleware_cfg = to_object(MiddlewareConfigHandler("mysql").get_all_config_with_check_result())
        for k, v in self.middleware_cfg.items():
            if not self.middleware_cfg[k]["ipaddress"]:
                self.middleware_cfg[k]["ipaddress"] = self.middleware_server_list
        self.middleware_cfg.update(self.check_result.__dict__)
        if "lang" in self.check_result.__dict__["config"]:
            self.middleware_cfg["lang"] = self.check_result.__dict__["config"]["lang"]

        if "deploy_https" in self.check_result.__dict__["config"]:
            if self.check_result.__dict__["config"]["deploy_https"]:
                self.middleware_cfg["scheme"] = "https"
            else:
                self.middleware_cfg["scheme"] = "http"
        if "jwt" in self.check_result.__dict__["config"]:
            self.middleware_cfg["jwt"] = self.check_result.__dict__["config"]["jwt"]
        if "processing_unit" in self.check_result.__dict__["config"]:
            self.middleware_cfg["processing_unit"] = self.check_result.__dict__["config"]["processing_unit"]
        self.servers = check_result.servers.get()
        self.service_info = service_path.config
        self.private_deploy_version = self.service_info.tag

        self.namespace = self.service_path.config.namespace
        self.namespaces = [self.namespace, "proxy"]

        self.project = self.namespace
        self.istio_injection_namespaces = [self.namespace, "proxy", ]
        if self.namespace == "license":
            self.istio_injection_namespaces.remove("license")

        self.templates_dir = self.service_path.template
        self.data_dir = self.service_path.data
        self.deploy_dir = self.check_result.deploy_dir

        self.service_charts_remote = path_join(self.deploy_dir, "chart")
        self.harbor_cfg = HarborConfigHandler().get_config_with_check_result()
        try:
            harbor_ip = self.harbor_cfg["harbor"]["lb"]
        except IndexError:
            harbor_ip = self.check_result.servers.get_role_ip("harbor")[0]
        self.registry_hub = "{}:{}".format(harbor_ip, self.harbor_cfg["harbor"]["nginx_harbor_proxy_port"])

        self.etcd_servers = self.check_result.servers.get_role_ip("master")
        self.etcd_endpoint = "\,".join(
            ["{}:{}".format(server, self.middleware_cfg.etcd.http_port) for server in self.etcd_servers]
        )

        self.env_k8s_config_src = path_join(self.templates_dir, "routes")
        self.env_k8s_config_dest = path_join(self.templates_dir, "routes_dest")
        self.env_k8s_config_remote = path_join(self.deploy_dir, "routes_dest")

        self.nginx_compose_file = path_join(self.deploy_dir, "nginx", "docker-compose.yml")

        self.servers = self.check_result.servers.get()
        self.middleware_cfg["k8s_hosts"] = [x.ipaddress for x in self.servers]
        self.master_hosts = self.check_result.servers.get_role_obj("master")
        self.middleware_cfg["k8s_masters"] = [x.ipaddress for x in self.master_hosts]

        self.replicas = 1
        self.nodes = self.check_result.servers.get_role_obj("node")
        self.master_host = self.check_result.servers.get_role_obj("master")[0]
        self.harbor_hosts = self.check_result.servers.get_role_obj("harbor")

        self.rabbitmq_init_file_template_path = path_join(self.templates_dir, "init_rabbitmq.tmpl")
        self.rabbitmq_init_file_path = path_join(self.templates_dir, "init_rabbitmq.sh")

        self.redis_init_file_template_path = path_join(self.templates_dir, "init_redis.tmpl")
        self.redis_init_file_path = path_join(self.templates_dir, "init_redis.sh")

        self.component = self.service_path.config.component
        self.init = self.service_path.config.init

        # bin
        self.fill_bin_src = path_join(self.templates_dir, "pvt_gen-linux-amd64")
        self.fill_bin_remote = path_join(self.deploy_dir, "pvt_gen-linux-amd64")
        # valuePath
        self.templates_src = path_join(self.templates_dir, "values")
        self.common_dest = path_join("/tmp", "{}_values".format(self.project))
        self.common_remote = path_join(self.deploy_dir, "{}_values".format(self.project))
        #tmplPath
        # self.config_templates = path_join(self.templates_dir,
        #                                   "{}_conf_templates".format(self.component),
        #                                   self.component.capitalize())
        #
        # self.config_remote = path_join(self.deploy_dir,
        #                                "{}_conf_templates".format(self.component),
        #                                self.component.capitalize())
        self.config_templates = path_join(self.templates_dir, "templates", self.component.capitalize())

        # self.config_remote = path_join(self.deploy_dir, "{}_conf_templates".format(self.component))
        self.config_remote = path_join(self.deploy_dir, "templates", self.component.capitalize())

        if self.component == "nlp":
            self.config_templates = path_join(self.templates_dir,
                                              "{}_conf_templates".format(self.component),
                                              "Mage")
            self.config_remote = path_join(self.deploy_dir,
                                           "{}_conf_templates".format(self.component),
                                           "Mage")
        # if self.component == "license":
        #     self.config_templates = path_join(self.templates_dir, "templates", "Mid")
        #     self.config_remote = path_join(self.deploy_dir, "templates", "Mid")

        # self.config_target = path_join(self.deploy_dir, "{}_configmap".format(self.component))
        self.config_target = path_join(self.deploy_dir, "configmap")
        # config原生配置
        # self.config = path_join(self.deploy_dir, "{}_config".format(self.component))
        self.config = path_join(self.deploy_dir, "{}_config".format(self.component))

        # configmap
        self.configmap = path_join(self.config_target, self.component.capitalize())
        self.configmap_remote = path_join(self.deploy_dir, self.component.capitalize())
        # if self.component == "nlp":
        #     self.configmap = path_join(self.config_target, "Mage")
        #     self.configmap_remote = path_join(self.deploy_dir, "Mage")

        # if self.component == "license":
        #     self.configmap = path_join(self.config_target, "Mid")
        #     self.configmap_remote = path_join(self.deploy_dir, "Mid")

        # 临时解决istio vs 命名空间问题
        # vs_namespaces = ["creativity", "entcmd", "internet", "istio-system", "mid", "proxy", "rpa-collaboration"]
        # for ns in vs_namespaces:
        #     log.info(Helper().CREATE_NAMESPACE.format(ns))
        #     cmd = CREATE_NS.format(ns)
        #     self._exec_command_to_host(cmd=cmd, server=self.master_host, check_res=False)

    def _fill_item_file(self):
        log.info(Helper().LOCAL_FILL.format(self.templates_src, self.common_dest))
        try:
            FileTemplate(self.middleware_cfg, self.templates_src, self.common_dest).fill()
        except UtilsError as e:
            log.error(e.msg)
            exit(e.code)
        return True if os.path.isdir(self.common_dest) else False

    def _generic_configmap(self, genProjects=""):
        """
        """
        if self._fill_item_file():
            self._send_file(src=self.fill_bin_src, dest=self.fill_bin_remote)
            self._send_file(src=self.common_dest, dest=self.common_remote)
            self._send_file(src=self.config_templates, dest=self.config_remote)
        # 检查是否存在 生成指定服务配置
            if genProjects:
                genProjects = "-genProjects={}".format(genProjects)

            cmd = [
                "chmod +x {}".format(self.fill_bin_remote),
                "{} -tmplPath={} -valuePath={} -targetPath={} -configTargetPath={} {}".format(
                    self.fill_bin_remote,
                    self.config_remote,
                    self.common_remote,
                    self.config_target,
                    self.config,
                    genProjects
                )
            ]
            # log.info(cmd)
            self._exec_command_to_host(cmd=cmd, server=self.servers[0])

    def deploy_all_configmap(self):
        service_list = []
        for i in self.service_path.config.services:
            service_list.append(i)
        # 处理 entuc entcmd
        service_list.append("entuc-config")
        service_list.append("entcmd-config")

        genProjects = ",".join(service_list)

        self._generic_configmap(genProjects)
        self._create_namespace(namespaces=self.namespaces, istio_injection_namespaces=self.istio_injection_namespaces)
        self._send_file(src=self.configmap, dest=self.configmap_remote)
        cmd = "kubectl apply -f {}".format(self.configmap_remote)
        self._exec_command_to_host(cmd=cmd, server=self.harbor_hosts[0])
        log.info(self.configmap)
        log.info(self.configmap_remote)

    def push_images(self, project):
        harbor = HarborModel(username="admin", host=self.registry_hub, password=self.middleware_cfg["harbor"]["password"])
        if project not in harbor.list_project():
            harbor.create_project(project)

        if os.path.exists(self.service_path.image):
            for image in os.listdir(self.service_path.image):
                image_path = path_join(self.service_path.image, image)
                log.info(Helper().PUSH_IMAGE.format(image_path))
                # print(image_path)
                docker = DockerImageModel(
                    image=image_path, project=project, repo=self.registry_hub, tag_version=self.private_deploy_version
                )
                docker.run()

    def _send_file(self, src, dest, servers=None):
        l = []
        if servers:
            if isinstance(servers, list):
                l = servers
            else:
                l.append(servers)
        else:
            l = self.servers

        for server in l:
            log.info(Helper().SEND_FILE.format(src, server.ipaddress, dest))
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            try:
                ssh_cli.put(src, dest)
            except Exception as e:
                log.error(e)
                exit(2)
            finally:
                ssh_cli.close()

    def _exec_command_to_host(self, cmd, server: ServerHandler, check_res=True) -> dict:
        log.info(Helper().EXCUTE_COMMAND.format(server.ipaddress, cmd))
        if isinstance(cmd, list):
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            res_list = ssh_cli.run_cmdlist(cmd)
            ssh_cli.close()
            if check_res:
                for res in res_list:
                    if res["code"] != 0:
                        log.error("{} {}".format(res["stdout"], res["stderr"]))
                        exit(2)
            return res_list
        if isinstance(cmd, str):
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            res = ssh_cli.run_cmd(cmd)
            ssh_cli.close()
            if check_res:
                if res["code"] != 0:
                    log.error("{} {}".format(res["stdout"], res["stderr"]))
                    exit(2)
            # print(res["stdout"])
            return res
        else:
            log.error(Helper().COMMAND_ERROR.format(cmd))
            exit(2)

    def _create_namespace(self, namespaces, istio_injection_namespaces):
        if namespaces:
            for ns in namespaces:
                log.info(Helper().CREATE_NAMESPACE.format(ns))
                cmd = CREATE_NS.format(ns)
                self._exec_command_to_host(cmd=cmd, server=self.master_host, check_res=False)
        if istio_injection_namespaces:
            for ns in istio_injection_namespaces:
                inject_cmd = ISTIO_INJECTION_NS.format(ns)
                self._exec_command_to_host(cmd=inject_cmd, server=self.master_host, check_res=False)

    def deploy_istio(self):
        # 渲染istio配置
        log.info(path_join(self.env_k8s_config_src, "istio"))
        log.info(path_join(self.env_k8s_config_dest, "istio"))
        FileTemplate(
            self.middleware_cfg,
            path_join(self.env_k8s_config_src),
            path_join(self.env_k8s_config_dest)
        ).fill()
        self._send_file(src=self.env_k8s_config_dest, dest=self.env_k8s_config_remote)
        # 临时解决entuc istio路由x-forwarded-proto: https问题
        if self.project == "entuc" or self.project == "entcmd" or self.project == "rpa-collaboration":
            sed_cmd = "sed -i '/headers/,+3d' {}".format(path_join(self.env_k8s_config_remote,
                                                                                 "entuc-http-vs.yaml"))
            self._exec_command_to_host(cmd=sed_cmd, server=self.master_host, check_res=False)

        cmd = KUBECTL_APPLY.format(path_join(self.env_k8s_config_remote))
        self._exec_command_to_host(cmd=cmd, server=self.master_host, check_res=False)

    def _create_logs_dir(self, project):
        log_path = os.path.join(self.deploy_dir, "Logs", project)
        log.info(Helper().CREATE_LOG_PATH.format(log_path))
        cmd = [
            MKDIR_DIR.format(log_path),
            CHMOD_777.format(log_path)
        ]
        for server in self.servers:
            self._exec_command_to_host(cmd=cmd, server=server)

    def _get_data_pvc_name(self, project, pvc_name):
        # 获取pvc的volumeName
        counter = 0
        succeed = False
        retry_count = 6
        retry_time_interval = 10
        pvc_volume_name = ""
        while not succeed and counter < retry_count:
            time.sleep(retry_time_interval)

            namespace = project
            cmd = GET_PVC_VOLUMENAME_CMD.format(namespace, pvc_name)
            res = self._exec_command_to_host(cmd=cmd, server=self.master_host, check_res=True)

            pvc_volume_name = res["stdout"].strip()
            if not pvc_volume_name:
                succeed = False
                counter += 1
            succeed = True

        return succeed, pvc_volume_name

    def _get_nfs_pvc_path(self):
        # 获取local-path pvc的volumeName
        namespace = "rook-nfs"
        local_pvc_name = "nfs-default-claim"
        cmd = GET_PVC_VOLUMENAME_CMD.format(namespace, local_pvc_name)
        res = self._exec_command_to_host(cmd=cmd, server=self.master_host, check_res=True)
        if res["code"] != 0:
            log.error(Helper().GET_LOCAL_PATH_PVC_ERROR.format(res["stdout"]))
            exit(2)
        local_pvc_volume_name = res["stdout"].strip()
        if not local_pvc_volume_name:
            log.error(Helper().GET_LOCAL_PATH_PVC_ERROR.format(res["stdout"]))
            exit(2)
        # 返回拼接目录
        return "{}_{}_{}".format(local_pvc_volume_name, namespace, local_pvc_name)

    def _get_nfs_node(self):
        cmd = GET_NFS_NODE_IP_CMD
        res = self._exec_command_to_host(cmd=cmd, server=self.master_host, check_res=True)
        if res["code"] != 0:
            log.error(Helper().GET_NFS_NODE_IP_ERROR.format(res["stdout"]))
            exit(2)
        nfs_node_ip = res["stdout"].strip()
        if not nfs_node_ip:
            log.error(Helper().GET_NFS_NODE_IP_ERROR.format(res["stdout"]))
            exit(2)
        return nfs_node_ip

    def prepare_data(self, project):
        if self.service_path.config.mount_data:
            for data_info in self.service_path.config.mount_data:
                data_src_dir = path_join(self.service_path.data, data_info["data_name"])

                pvc_name = "{}-{}-{}-claim".format(data_info["service_name"], data_info["process_name"], data_info["relative_path"])
                succeed, pvc_volume_name = self._get_data_pvc_name(project=project, pvc_name=pvc_name)
                if not succeed:
                    log.error(Helper().CREATE_PVC_ERROR)
                    exit(2)

                if self.check_result.use_external_disk:
                    # rook-ceph远程位置
                    use_nfs = False
                    data_remote = path_join(
                        "/var/lib/kubelet/plugins/kubernetes.io/csi/pv",
                        pvc_volume_name, "globalmount"
                    )
                else:
                    # rook-nfs远程位置
                    use_nfs = True
                    local_pvc_path = self._get_nfs_pvc_path()
                    data_remote = path_join(
                        self.deploy_dir, "nfs",
                        local_pvc_path,
                        "{}-{}-{}".format(project, pvc_name, pvc_volume_name)
                    )
                # 发送文件
                for model_dir in os.listdir(data_src_dir):
                    for i in os.listdir(path_join(data_src_dir, model_dir)):
                        model_dir_path = path_join(data_src_dir, model_dir, i)
                        data_remote_path = path_join(data_remote, i)
                        if use_nfs:
                            # nfs_node_ip = self._get_nfs_node()
                            for s in self.servers:
                                # if s.ipaddress == nfs_node_ip:
                                self._send_file(src=model_dir_path, dest=data_remote_path, servers=s)
                                # 解决挂载属主属组权限问题
                                chown_cmd = "chown -R 5000:5000 {}".format(data_remote)
                                self._exec_command_to_host(cmd=chown_cmd, server=s, check_res=True)
                        else:
                            self._send_file(src=model_dir_path, dest=data_remote_path)

                # 重新启动
                time.sleep(60)
                namespace = project
                restart_deployment_cmd = RESTART_DEPLOYMENT.format(namespace, data_info["process_name"])
                self._exec_command_to_host(cmd=restart_deployment_cmd, server=self.master_host, check_res=True)

    def start_service(self, project, version, tf_name="mage"):
        self._send_file(src=self.service_path.charts, dest=self.service_charts_remote)
        if self.check_result.use_external_disk:
            external_storageclass_name = "rook-cephfs"
        else:
            external_storageclass_name = "rook-nfs-share1"

        if self.service_path.config.services:
            for service, processes in self.service_path.config.services.items():
                for process in processes:
                    log.info(Helper().DEPLOY_SERVICE.format(process))
                    check_cmd = HELM_LIST.format(process)
                    check_results = self._exec_command_to_host(cmd=check_cmd, server=self.master_host, check_res=False)
                    is_exist = False
                    if check_results["stdout"]:
                        chart_dict = json.loads(check_results["stdout"])
                        chart_list = []
                        for i in chart_dict["Releases"]:
                            chart_list.append(i["Name"])
                        if process in chart_list:
                            log.warning(Helper().SERVICE_EXISTS.format(process))
                            is_exist = True

                    if not is_exist:
                        self._create_logs_dir(service)
                        lm_client_service_id = ""
                        if process in self.admin_config.license_service:
                            lm_client_service_id = self.admin_config.license_service[process]

                        registry_hub = path_join(self.registry_hub, project)
                        image_name = process
                        tag = version
                        workload_type=""
                        file_path = os.path.join(self.service_charts_remote, process)
                        logpath = "/home/works/program/logs"

                        if self.service_path.config.machine_type == "cpu" and self.service_path.config.require_tfserver:
                            resources = ""
                            registry_hub = path_join(self.registry_hub, "middleware")
                            image_name = "tfserving"
                            tag = "2.4.0-cpu" if self.service_path.config.machine_type == "cpu" else "2.8.0-gpu-encrypted"
                            cmd = HELM_INSTALL_CPU.format(
                                process=process,
                                process_name=process,
                                project_name=service,
                                namespace=project,
                                component=self.component,
                                logpath=logpath,
                                external_storageclass_name=external_storageclass_name,
                                replicas=self.replicas,
                                registry_hub=registry_hub,
                                image_name=image_name, image_tag=tag,
                                pvt_work_dir=self.deploy_dir,
                                etcd_endpoint=self.etcd_endpoint,
                                lm_client_service_id=lm_client_service_id,
                                workload_type=workload_type,
                                resources=resources,
                                file_path=file_path
                            )
                        elif self.service_path.config.machine_type == "gpu":
                            registry_hub = path_join(self.registry_hub, "middleware")
                            image_name = "tfserving"
                            tag = "2.4.0-cpu" if self.service_path.config.machine_type == "cpu" else "2.8.0-gpu-encrypted"

                            cmd = HELM_INSTALL_GPU.format(
                                process=process,
                                process_name=process,
                                project_name=service,
                                namespace=project,
                                component=self.component,
                                logpath=logpath,
                                external_storageclass_name=external_storageclass_name,
                                replicas=self.replicas,
                                registry_hub=registry_hub,
                                image_name=image_name, image_tag=tag,
                                pvt_work_dir=self.deploy_dir,
                                etcd_endpoint=self.etcd_endpoint,
                                lm_client_service_id=lm_client_service_id,
                                workload_type=workload_type,
                                file_path=file_path
                            )
                        else:
                            resources = ""
                            cmd = HELM_INSTALL_CPU.format(
                                process=process,
                                process_name=process,
                                project_name=service,
                                namespace=project,
                                component=self.component,
                                logpath=logpath,
                                external_storageclass_name=external_storageclass_name,
                                replicas=self.replicas,
                                registry_hub=registry_hub,
                                image_name=image_name, image_tag=tag,
                                pvt_work_dir=self.deploy_dir,
                                etcd_endpoint=self.etcd_endpoint,
                                lm_client_service_id=lm_client_service_id,
                                workload_type=workload_type,
                                resources=resources,
                                file_path=file_path
                            )
                        self._exec_command_to_host(cmd=cmd, server=self.master_host)


    def upgrade_service(self, project):
        for service, processes in self.service_path.config.services.items():
            for process in processes:
                log.info(Helper().DEPLOY_SERVICE.format(process))
                self._create_logs_dir(service)
                file_path = os.path.join(self.service_path.charts, process)
                # print(file_path)
                cmd = HELM_UPGRADE_INSTALL.format(
                    process=process, replicas=self.replicas,
                    registry_hub=path_join(self.registry_hub, project),
                    image_name=process, image_tag=self.private_deploy_version,
                    pvt_work_dir=self.deploy_dir,
                    etcd_endpoint=self.etcd_endpoint,
                    file_path=file_path)
                self._exec_command_to_host(cmd=cmd, server=self.master_host)

    def proxy_on_nginx(self, nginx_template, nginx_tmp, nginx_file_remote):
        FileTemplate(self.middleware_cfg, nginx_template, nginx_tmp).fill()
        self._send_file(src=nginx_tmp, dest=nginx_file_remote)
        compose_cmd = ComposeModel(self.nginx_compose_file)
        for server in self.servers:
            self._exec_command_to_host(cmd=compose_cmd.restart(), server=server, check_res=True)

    def init_identity_user(self):
        init_user_cmd = INIT_IDENTITY_USER
        self._exec_command_to_host(cmd=init_user_cmd, server=self.servers[0], check_res=False)

    def init_usercenter_user(self):
        init_user_url = "http://{}:{}/api/tenant/install/byci".format(self.middleware_cfg.nginx.lb,
                                                                      self.middleware_cfg.nginx.entuc_proxy_port)
        log.info(init_user_url)
        headers = {
            # 'Content-Type': 'application/json',
        }
        json_data = {'username': 'admin', 'password': 'Abc123456', 'tenantName': 'Laiye Tech',}
        counter = 0
        succeed = False
        msg = ""
        while not succeed and counter < 3:
            time.sleep(10)
            try:
                response = requests.post(init_user_url, headers=headers, json=json_data, timeout=100)
                if response.json()["code"] == 102 or response.json()["code"] == 0:
                    succeed = True
                    msg = response.json()
                    log.info(Helper().INIT_ENTUC_USER_SUCCEED.format(msg))
                else:
                    counter = counter + 1
                    msg = response.json()
                    log.error(Helper().INIT_ENTUC_USER_FAILED.format(msg))
            except Exception as e:
                counter = counter + 1
                msg = "Access Error: {}".format(e)
                log.error(Helper().INIT_ENTUC_USER_FAILED.format(msg))
        if succeed:
            log.info(Helper().INIT_ENTUC_USER_SUCCEED.format(msg))
        else:
            log.error(Helper().INIT_ENTUC_USER_FAILED.format(msg))
            exit(2)

    def init_usercenter_clients(self, project):
        project_dict = {
            "entcmd": {
                "ClientId": "uibot.commander",
                "RegisterSecret": "Z5p1XuQxqdl2RMcyJ9u3gWR00mj4tYmX",
                "ClientBaseUri": "http://{}:{}".format(self.middleware_cfg.nginx.lb,
                                                       self.middleware_cfg.nginx.entcmd_proxy_port),
                "ClientSecret": "{}".format(self.middleware_cfg.jwt)
            },
            "mage": {
                "ClientId": "uibot.mage",
                "RegisterSecret": "1ZuKG91AIw700rAkTxWu0bzCNemTpXVu",
                "ClientBaseUri": "http://{}:{}".format(self.middleware_cfg.nginx.lb,
                                                       self.middleware_cfg.nginx.mage_proxy_port),
                "ClientSecret": "laiye"
            },
            "chatbot": {
                "ClientId": "laiye.chatbot",
                "RegisterSecret": "EuktBwJP5h4a9AUiZn5dlauF3UpOupAD",
                "ClientBaseUri": "http://{}:{}".format(self.middleware_cfg.nginx.lb,
                                                       self.middleware_cfg.nginx.chatbot_proxy_port),
                "ClientSecret": "laiye"
            },
            "rpa-collaboration": {
                "ClientId": "uibot.collaboration",
                "RegisterSecret": "RK6XOr2WdjZXQA5RuuTHYfwhER8EppRi",
                "ClientBaseUri": "http://{}:{}".format(self.middleware_cfg.nginx.lb,
                                                       self.middleware_cfg.nginx.rpa_collaboration_backend_proxy_port),
                "ClientSecret": "laiye666"
            },
            "license": {
                "ClientId": "license.manager",
                "RegisterSecret": "nJNfz4SBWL481XfacQSSgtcayAqXwqVA",
                "ClientBaseUri": "http://{}:{}".format(self.middleware_cfg.nginx.lb,
                                                        self.middleware_cfg.nginx.license_web_proxy_port),
                "ClientSecret": "laiye"
            }
        }

        init_clients_url = "http://{}:{}/.well-known/service-register-v6".format(
            self.middleware_cfg.nginx.lb,
            self.middleware_cfg.nginx.entuc_proxy_port)
        headers = {
            'Content-Type': 'application/json',
        }

        json_data = {
            "clientId": project_dict[project]["ClientId"],
            "registerSecret": project_dict[project]["RegisterSecret"],
            "ClientBaseUri": project_dict[project]["ClientBaseUri"],
            "ClientSecret": project_dict[project]["ClientSecret"]
        }

        try:
            response = requests.post(init_clients_url, headers=headers, json=json_data, timeout=100)
            if response.status_code == 200 and response.json()["code"] == 0:
                log.info(Helper().CONFIG_CLIENTS_SUCCEED.format(response.json()["message"]))
            else:
                log.error(Helper().CONFIG_CLIENTS_FAILED.format(response.status_code))
                log.error(response.json())
                log.error(response.json()["message"])
                exit(2)
        except Exception as e:
            log.error(Helper().CONFIG_CLIENTS_FAILED.format(e))
            exit(2)

    def deploy_tf_service(self, module_name, tf_image_name):
        # self._send_file(src=self.service_path.charts, dest=self.service_charts_remote)
        log.info(Helper().DEPLOY_SERVICE.format(module_name))
        check_cmd = HELM_LIST.format(
            module_name, module_name
        )
        check_results = self._exec_command_to_host(cmd=check_cmd, server=self.master_host, check_res=False)
        if check_results["code"] == 0:
            log.warning(Helper().SERVICE_EXISTS.format(module_name))
        else:
            file_path = os.path.join(self.service_charts_remote, module_name)
            # print(file_path)
            tag = "2.4.0-cpu" if self.service_path.config.machine_type == "cpu" else "2.8.0-gpu-encrypted"
            cmd = HELM_INSTALL_TF_SERVICE.format(
                process=module_name, replicas=self.replicas,
                registry_hub=path_join(self.registry_hub, "middleware"),
                image_name=tf_image_name, image_tag=tag,
                pvt_work_dir=self.deploy_dir,
                etcd_endpoint=self.etcd_endpoint,
                file_path=file_path)

            self._exec_command_to_host(cmd=cmd, server=self.master_host)

    def project_pod_check(self):
        counter = 0
        succeed = False
        kube = KubeModel()

        # 每10秒重试一次，重试 10 次，如果还是不成功就报错
        while not succeed and counter < 10:
            time.sleep(10)
            pod_info = kube.get_all_pod_status(namespace=self.project)
            pod_status = []
            for pod in pod_info:
                pod_status.append(pod["status"])

            if set(pod_status) == {'Running', 'Succeeded'} or set(pod_status) == {'Running'}:
                succeed = True
            else:
                succeed = False
                counter = counter + 1
        if succeed:
            log.info(Helper().PROJECT_POD_CHECK_SUCCEED.format(self.project))
        else:
            log.info(Helper().PROJECT_POD_CHECK_FAILED.format(self.project))

    def app_test(self, project):
        if os.path.exists(self.service_path.test):
            time.sleep(120)
            # 导入测试镜像
            test_images_path = path_join(self.service_path.test, "images")
            for image in os.listdir(test_images_path):
                image_path = path_join(test_images_path, image)
                log.info(Helper().PUSH_IMAGE.format(image_path))
                # print(image_path)
                docker = DockerImageModel(image=image_path, project=project,
                                          repo=self.registry_hub, tag_version=self.private_deploy_version)
                docker.run()

            # 渲染测试配置文件
            config_path_src = path_join(self.service_path.test, "templates")
            config_path_dest = path_join(self.deploy_dir, "{}_test_config".format(project))
            # log.info("渲染填充项文件{}到{}".format(config_path_src, config_path_dest))
            try:
                FileTemplate(self.middleware_cfg, config_path_src, config_path_dest).fill()
            except Exception as e:
                log.error(Helper().FILL_ERROR.format(config_path_src))
                log.error(e)
                exit(2)

            # 运行测试命令
            host_config = os.path.join(config_path_dest, "test.conf")
            container_config = "/home/works/program/conf/test.conf"
            host_logs = path_join(self.deploy_dir, "{}_test_logs".format(project))
            container_logs = "/home/works/program/logs"
            image_name = path_join(self.registry_hub, project, "{}-apptest".format(project))

            cmd = TEST_SERVICE_CMD.format(
                self.middleware_cfg.mysql.lb,
                host_config, container_config,
                host_logs, container_logs,
                image_name, self.private_deploy_version
            )

            counter = 0
            succeed = False

            # 如果自动化测试失败，重试1次，如果还是不成功就报错
            while not succeed and counter < 2:
                time.sleep(10)

                log.info(cmd)
                code, res = runcmd(cmd)
                if code != 0:
                    log.error(res)
                    exit(2)

                # 判断测试结果
                res_json_file = path_join(host_logs, "results.json")
                with open(res_json_file, "r") as fp:
                    res = json.load(fp)
                if res["code"] != 0:
                    counter = counter + 1
                    succeed = False
                    log.error(Helper().APPTEST_FAILED.format(project, res["msg"]))
                    break
                else:
                    succeed = True
                    log.info(Helper().APPTEST_SUCCEED.format(project, res["msg"]))
                    return

    def init_mysql(self):
        if self.init:
            if self.init["mysql_db"]:
                if self.middleware_cfg.mysql.is_deploy:
                    mysql_host = self.master_host.ipaddress
                else:
                    mysql_host = self.middleware_cfg.mysql.ipaddress[0]
                sql = SqlModule(host=mysql_host, port=int(self.middleware_cfg.mysql.port),
                                user=self.middleware_cfg.mysql.username, passwd=self.middleware_cfg.mysql.password)
                for db in self.init["mysql_db"]:
                    create_db = CREATE_DB.format(db_name=db)
                    sql.insert_sql(create_db)

    def init_minio(self):
        if self.init:
            if "minio_bucket_public" in self.init:
                endpoint = "{}:{}".format(self.middleware_cfg.minio.lb,
                                          self.middleware_cfg.minio.nginx_proxy_port)
                cli = Minio(
                    endpoint,
                    self.middleware_cfg.minio.username,
                    self.middleware_cfg.minio.password,
                    secure=False
                )
                for bucket in self.init["minio_bucket_public"]:
                    if not cli.bucket_exists(bucket):
                        cli.make_bucket(bucket)
                    policy_read_write = {
                        "Version": "2012-10-17",
                        "Statement": [
                            {
                                "Action": [
                                    "s3:GetObject"
                                ],
                                "Effect": "Allow",
                                "Principal": "*",
                                "Resource": [
                                    "arn:aws:s3:::{}/*".format(bucket)
                                ],
                                "Sid": ""
                            }
                        ]
                    }
                    cli.set_bucket_policy(bucket, json.dumps(policy_read_write))
            if "minio_bucket_private" in self.init:
                endpoint = "{}:{}".format(self.middleware_cfg.minio.lb,
                                          self.middleware_cfg.minio.nginx_proxy_port)
                cli = Minio(
                    endpoint,
                    self.middleware_cfg.minio.username,
                    self.middleware_cfg.minio.password,
                    secure=False
                )
                for bucket in self.init["minio_bucket_private"]:
                    if not cli.bucket_exists(bucket):
                        cli.make_bucket(bucket)
                    policy_read_write = {
                        "Version": "2012-10-17",
                        "Statement": [
                            {
                                "Action": [
                                    "s3:GetObject"
                                ],
                                "Effect": "Allow",
                                "Principal": "*",
                                "Resource": [
                                    "arn:aws:s3:::{}/*".format(bucket)
                                ],
                                "Sid": ""
                            }
                        ]
                    }
                    cli.set_bucket_policy(bucket, json.dumps(policy_read_write))

    def init_rabbitmq(self):
        if self.init:
            if self.init["rabbitmq_vhost"]:
                for vhost in self.init["rabbitmq_vhost"]:
                    rabbitmq_api_vhosts = "http://{}:{}/api/vhosts/{}".format(self.master_host.ipaddress,
                                                                              self.middleware_cfg.rabbitmq.manage_port,
                                                                              vhost)
                    res = requests.put(url=rabbitmq_api_vhosts, auth=(self.middleware_cfg.rabbitmq.username,
                                                                      self.middleware_cfg.rabbitmq.password))
                    if res.status_code > 204:
                        log.error("rabbitmq vhost error.")
                        exit(2)



import os

from laipvt.handler.confighandler import CheckResultHandler
from laipvt.sysutil.gvalue import CHECK_FILE, LAIPVT_BASE_DIR, LOG_TO_TTY
from laipvt.sysutil.log import Logger
from laipvt.sysutil.ssh import SSHConnect


class DeletePackageHandler:
    """ 删除程序功能 """

    def __init__(self):
        # 获取前置检查结果
        self.check_result_file = CHECK_FILE
        self.check_result = CheckResultHandler(self.check_result_file)
        self.deploy_dir = self.check_result.deploy_dir
        self.servers = self.check_result.servers.servers
        self.servers_all_ip = self.check_result.servers.get_all_ip()
        self.python_path = "/usr/local/python-3.7.6"
        self.laipvt_base_path = LAIPVT_BASE_DIR
        self.laipvt_middleware_path = "{}/middleware".format(self.laipvt_base_path)
        self.tmp_path = "/tmp/rpa"

    def stopserver(self, server: str):
        """ 停止服务 """
        cmd = "systemctl stop {server}".format(server=server)
        return cmd

    def removedir(self, dir_path: str):
        """ 移动目录到/tmp/rpa下 """
        cmd = "mv {dir_path} {tmp}".format(dir_path=dir_path, tmp=self.tmp_path)
        return cmd

    def deldir(self, dir_path: str):
        """ 删除目录到/tmp/rpa下 """
        cmd = "rm -rf {dir_path}".format(dir_path=dir_path)
        return cmd

    def downserver(self, yaml_path: str):
        """ Down服务 """
        cmd = "docker-compose -f {path} down".format(path=yaml_path)
        return cmd

    def checkdirpath(self, dir_path: str):
        """ 检查目录是否存在 """
        if os.path.isdir(dir_path):
            return True
        return False

    def sshclient(self, cmd):
        """ 直接执行命令 """
        for server in self.servers:
            ip = server.d["ipaddress"]
            username = server.d["username"]
            password = server.d["password"]
            port = server.d["port"]
            ssh_cli = SSHConnect(hostip=ip, username=username, password=password, port=port)
            self.log.info("{} 执行 {} 命令".format(ip, cmd))
            ssh_cli.run_cmd(cmd)
            ssh_cli.close()


class DeleteMiddleware(DeletePackageHandler):
    """ 移除各中间件服务 """

    def __init__(self):
        super(DeleteMiddleware, self).__init__()
        self.log = Logger(
                tty=os.environ.get("LOG_TO_TTY", LOG_TO_TTY),
                log_name="delete_middleware.log",
                log_path="/tmp/log",
        ).get()

    def get_all_middleware_name(self):
        """ 获取所有中间件的名称 """
        _path = self.laipvt_middleware_path
        if self.checkdirpath(_path):
            middleware = [os.path.splitext(fn)[0] for fn in os.listdir(_path)]
        return middleware

    def get_dockercompose_yaml(self, middleware_name):
        """ 获取docker-compose文件 """
        middleware_yaml = "{}/{}/{}".format(self.deploy_dir, middleware_name, "docker-compose.yml")
        return middleware_yaml

    def run(self):
        self.log.info("开始清除Middleware环境")
        middlewares = self.get_all_middleware_name()
        for name in middlewares:
            yaml_path = self.get_dockercompose_yaml(name)
            downcmd = self.downserver(yaml_path=yaml_path)
            mvcmd = self.deldir("{}/{}".format(self.deploy_dir, name))
            cmd = "{downcmd};{mvcmd}".format(downcmd=downcmd, mvcmd=mvcmd)
            self.sshclient(cmd)
        self.log.info("Middleware环境清理完成")


class DeleteKubernetes(DeletePackageHandler):
    """ 移除k8s """

    def __init__(self):
        super(DeleteKubernetes, self).__init__()
        self.kubernetes_path = ["/etc/kubernetes", "/var/lib/kubelet", "/var/lib/etcd", "~/.kube"]
        self.kubelet = "kubelet"
        self.tiller = "tiller"
        self.kubeadm_reset = "kubeadm reset -f"
        self.log = Logger(
                tty=os.environ.get("LOG_TO_TTY", LOG_TO_TTY),
                log_name="delete_k8s.log",
                log_path="/tmp/log",
        ).get()

    def run(self):
        self.log.info("开始清除Kubernetes环境")
        self.sshclient(self.kubeadm_reset)
        for path in self.kubernetes_path:
            cmd = self.deldir(path)
            self.sshclient(cmd)
        cmd = """cat > /tmp/clear_k8s << EOF
kubeadm reset -f
systemctl stop kubelet 
systemctl stop tiller
rm -rf /var/lib/cni
rm -rf /var/lib/kubelet
rm -rf /etc/cni/
ifconfig cni0 down
ifconfig flannel.1 down
ifconfig docker0 down
ip link delete cni0
ip link delete flannel.1
rm -rf /usr/lib/systemd/system/kubelet.service.d
rm -rf /opt/cni/bin
rm -rf /usr/bin/kubeadm
rm -f /lib/systemd/system/kubelet.service
EOF"""
        self.sshclient(cmd)
        self.sshclient("bash /tmp/clear_k8s && rm -f /tmp/clear_k8s")
        self.log.info("Kubernetes环境清理完成")


class DeleteDocker(DeletePackageHandler):
    """ 停止Docker并移除Docker """

    def __init__(self):
        super(DeleteDocker, self).__init__()
        self.docker_path = "{}/{}".format(self.deploy_dir, "Docker")
        self.log = Logger(
                tty=os.environ.get("LOG_TO_TTY", LOG_TO_TTY),
                log_name="delete_docker.log",
                log_path="/tmp/log",
        ).get()

    def run(self):
        self.log.info("开始清除Docker环境")
        cmd = """cat > /tmp/clear_docker << EOF
systemctl stop docker
rm -rf /var/lib/docker
ip link delete docker0
ip link delete kube-ipvs0
rm -f /usr/lib/systemd/system/containerd.service
rm -f /usr/lib/systemd/system/docker.socket
rm -f /usr/lib/systemd/system/docker.service
rm -f /usr/bin/docker
EOF"""
        self.log.info(cmd)
        self.sshclient(cmd)
        self.sshclient("bash /tmp/clear_docker && rm -f /tmp/clear_docker")
        self.log.info("Docker环境清理完成")


class DeleteAll(DeletePackageHandler):
    def __init__(self):
        super(DeleteAll, self).__init__()
        self.log = Logger(
                tty=os.environ.get("LOG_TO_TTY", LOG_TO_TTY),
                log_name="delete_all.log",
                log_path="/tmp/log",
        ).get()

    def Clear(self):
        """ 移除整个部署目录 """
        cmd1 = "{}".format(self.deldir(self.deploy_dir))
        self.log.info(cmd1)
        self.sshclient(cmd1)
        cmd2 = "{};{};{}".format(self.deldir(self.laipvt_base_path), self.deldir(self.python_path), self.deldir("/tmp/log"))
        self.log.info(cmd2)
        self.sshclient(cmd2)

    def run(self):
        """ 移除所有目录 """
        DeleteMiddleware().run()
        DeleteKubernetes().run()
        DeleteDocker().run()
        self.Clear()


def delete_main(args):
    if args.Middleware:
        """ 停止中间件服务并移除中间件目录 """
        DeleteMiddleware().run()

    if args.Docker:
        """ 移除Docker并停止Docker服务 """
        DeleteDocker().run()

    if args.Kubernetes:
        """ 移除K8S环境，并停止k8s服务 """
        DeleteKubernetes().run()

    if args.All:
        """ 移除所有环境 """
        DeleteAll().run()


if __name__ == "__main__":
    def delete_main(args):
        if args.Middleware:
            """ 停止中间件服务并移除中间件目录 """
            DeleteMiddleware().run()

        if args.Docker:
            """ 移除Docker并停止Docker服务 """
            DeleteDocker().run()

        if args.Kubernetes:
            """ 移除K8S环境，并停止k8s服务 """
            DeleteKubernetes().run()

        if args.All:
            """ 移除所有环境 """
            DeleteAll().run()



import os
from laipvt.sysutil.gvalue import LAIPVT_BASE_DIR, CHECK_FILE, PRE_CHECK_RESULT_FILE
from laipvt.sysutil.util import get_yaml_config

def is_pre_check():
    try:
        if os.path.exists(LAIPVT_BASE_DIR) and os.path.exists(CHECK_FILE) and os.path.exists(PRE_CHECK_RESULT_FILE):
            check_context = get_yaml_config(PRE_CHECK_RESULT_FILE)
            # print(check_context)
            if check_context["total"]:
                return True
        return False
    except Exception as e:
        return False


def check_use_https_ca():
    try:
        check_info = get_yaml_config(CHECK_FILE)

        if check_info["deploy_https"]:
            if not check_info["self_signed_ca"]:
                if not os.path.exists(check_info["ca_crt_path"]) and not os.path.exists(check_info["ca_key_path"]):
                    return False
        return True
    except Exception as e:
        return False

def check_https_ca_self_signed():
    try:
        check_info = get_yaml_config(CHECK_FILE)
        if check_info["self_signed_ca"]:
            return True
    except Exception as e:
        return False


from __future__ import absolute_import
from __future__ import unicode_literals

import pymongo
import requests
from laipvt.interface.middlewareinterface import MiddlewareInterface
from laipvt.handler.middlewarehandler import SiberConfigHandler
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.sysutil.template import FileTemplate
from laipvt.sysutil.util import path_join, ssh_obj, log, status_me
from laipvt.model.cmd import ComposeModel
from laipvt.sysutil.siber_main import Mage_test
from laipvt.helper.errors import Helper


class SiberController(MiddlewareInterface):
	def __init__(self, result: CheckResultHandler, handler: SiberConfigHandler, template: str):
		super(SiberController, self).__init__(result, handler, template)
		self.conf_tmp = path_join("/tmp")
		self.conf_template = path_join(self.template)
		self.conf_file = path_join(self.base_dir, "config")
		self.list_conf = ("siber.conf", "siber-web.conf", "siber-nginx.conf")
		self.mongo_data_source = path_join(self.template, "siber_mongo")
		self.mongo_data_remote = path_join(self.base_dir, "siber_mongo")
		self.siber_cfg = SiberConfigHandler().get_all_config_with_check_result()
		self.siber_cfg["siber"] = SiberConfigHandler().get_config_with_check_result()["siber"]
		self.siber_cfg["siber"]["ipaddress"] = self.handler.cfg["ipaddress"]
		self.siber_host = self.master_server[0].ipaddress
		self.tagid_list = []
		self.casid_list = []
		self.casid_basic_list = []

	def _generic_config(self):
		# log.info("渲染 Siber 配置文件")
		for num_id in range(len(self.list_conf)):
			dest = path_join(self.conf_template, self.list_conf[num_id])
			src_tmp = path_join(self.conf_tmp, self.list_conf[num_id])
			src = path_join(self.conf_file, self.list_conf[num_id])
			FileTemplate(self.siber_cfg, dest, src_tmp).fill()
			self.send_config_file(self.master_server[0], src_tmp, src)
		self.send_config_file(self.master_server[0], self.mongo_data_source, self.mongo_data_remote)
		self.generate_docker_compose_file(self.siber_cfg)
		self._send_docker_compose_file(self.master_server[0])

	def _send_docker_compose_file(self, host):
		log.info(Helper().SEND_FILE.format(self.docker_compose_file_tmp, host.ipaddress, self.docker_compose_file))
		ssh_cli = ssh_obj(ip=host.ipaddress, user=host.username, password=host.password, port=host.port)
		try:
			ssh_cli.put(self.docker_compose_file_tmp, self.docker_compose_file)
		except Exception as e:
			log.error(e)
			exit(2)
		finally:
			ssh_cli.close()

	def _init_siber(self):
		super().wait_for_service_start()
		log.info(Helper().INIT_SIBER_MONGO_DATA)
		try:
			init_mongo_cmd = "docker exec siber-mongo mongorestore -u admin -p admin -d admin siber_mongo/siber"
			server = self.master_server[0]
			ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
			res = ssh_cli.run_cmd(init_mongo_cmd)
			if res["code"] != 0:
				log.error(Helper().INIT_SIBER_MONGO_DATA_ERROR.format(res["stderr"]))
		except Exception as e:
			log.error(Helper().INIT_SIBER_MONGO_DATA_ERROR.format(e))

	def _start(self):
		compose_cmd = ComposeModel(self.docker_compose_file)
		for server in self.master_server[:1]:
			log.info(Helper().START_MIDDLEWARE_SERVICE.format(server.ipaddress, self.middleware_name))
			ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
			try:
				res = ssh_cli.run_cmd(compose_cmd.up())
				if res["code"] != 0:
					log.error(Helper().START_MIDDLEWARE_SERVICE_ERROR.format(res["stderr"], res["stdout"]))
			except Exception as e:
				log.error(e)
			finally:
				ssh_cli.close()

	def _check(self):
		super().wait_for_service_start()
		for server in self.master_server[:1]:
			res = self.check_port(server.ipaddress, self.siber_cfg["siber"]["port"])
			if res:
				log.info(Helper().CHECK_MIDDLEWARE_SERVICE_PORT_SUCCEED.format(server.ipaddress, self.siber_cfg["siber"]["port"]))
			else:
				log.error(Helper().CHECK_MIDDLEWARE_SERVICE_PORT_FAILED.format(server.ipaddress, self.siber_cfg["siber"]["port"]))

	def send_request(self, url, data):
		resp = requests.post(url=url, json=data)
		result = resp.json()
		return result

	def run_plan(self, plan_id):
		data = {
			"plan_info": {
				"plan_id": plan_id
			},
			"trigger_condition": {
				"environment_name": "prod"
			}
		}
		url = "http://{}:{}/siberhttp/run/plan".format(self.siber_host, self.siber_cfg["siber"]["port"])

		return self.send_request(url, data)

	def update_planversion(self, plan_version):
		data = {
			"manage_mode": "UPDATE",
			"plan_info": {
				"plan_name": "私有部署定制化mage plan",
				"interface_type": "http",
				"environment_name": "测试环境",
				"environment_id": "5e940f347dcd1000016d8378",
				"threads": 0,
				"flow_list": [
					"5f1926a438cb970001f5a936"
				],
				"plan_id": "5f1926b438cb970001f5a93d",
				"version_control": plan_version
			}
		}
		url = "http://{}:{}/siberhttp/manage/plan".format(self.siber_host, self.siber_cfg["siber"]["port"])
		return self.send_request(url, data)

	def update_basic_planversion(self, plan_version):
		data = {
			"manage_mode": "UPDATE",
			"plan_info": {
				"plan_name": "【Mage】Basic测试",
				"interface_type": "http",
				"environment_name": "测试环境",
				"environment_id": "5e940f347dcd1000016d8378",
				"threads": 0,
				"flow_list": [
					"60f140a5300d210001320076"
				],
				"plan_id": "5f8ecd1cb2b9fd0001c31217",
				"version_control": plan_version
			}
		}
		url = "http://{}:{}/siberhttp/manage/plan".format(self.siber_host, self.siber_cfg["siber"]["port"])
		return self.send_request(url, data)

	def replace_mage_collection_tag(self, tag_id):
		'''
		根据tag列表到mongo中取出tagid
		根据tagid将case取出
		将case到如到指定的flow中
		'''
		try:
			self.mongo_client = pymongo.MongoClient("mongodb://{}:27027/".format(self.siber_host))
			self.my_db = self.mongo_client.admin
			self.auth = self.my_db.authenticate("admin", "admin", mechanism='SCRAM-SHA-1')
			for tag_id in self.my_db.collection_tag.find({"tagname": {"$in": tag_id}}):
				self.tagid_list.append(tag_id["tagid"])
			for cas_tags in self.tagid_list:
				for res in self.my_db.collection_case.find({"$and": [{"casetags": cas_tags}, {"invaliddate": 0}]}):
					if "BASIC" not in res["casename"]:
						self.casid_list.append(res["_id"])
					elif "BASIC" in res["casename"]:
						self.casid_basic_list.append(res["_id"])
			self.my_db.collection_flow.update({"flowid": "5f1926a438cb970001f5a936"}, {"$set": {"caselist": self.casid_list}})
			self.my_db.collection_flow.update({"flowid": "60f140a5300d210001320076"}, {"$set": {"caselist": self.casid_basic_list}})
			self.run_mage_siber_test()
		except Exception as e:
			log.error(e)
			exit(2)

	def run_mage_siber_test(self):
		try:
			# log.info("替换 Siber env,oss 配置")
			Mage_test(self.siber_host).replace_private_oss_url()
			Mage_test(self.siber_host).update_private_mage_env_info()
			self.update_planversion("v1.15")
			self.update_basic_planversion("v1.15")
			self.run_plan("5f1926b438cb970001f5a93d")
			self.run_plan("5f8ecd1cb2b9fd0001c31217")
			log.info(Helper().SIBER_CHECK_INFO.format(self.siber_host))
		except Exception as e:
			log.error(e)
			exit(2)

	@status_me("middleware")
	def deploy_siber(self):
		self._generic_config()
		self._start()
		self._init_siber()
		self._check()



from __future__ import absolute_import
from __future__ import unicode_literals

from laipvt.sysutil.conf import YamlConfig
from laipvt.sysutil.gvalue import PORT_MAP, DEPLOY_LANGUAGE

module_require_tfserver = ("ocr_standard", "captcha", "nlp", "ocr_seal")
tfserver_name = {
    "ocr_standard": ["ocr-ctpn-tf-server", "ocr-text-recognition-tf-server",
                     "ocr-unet-table-tf-serving", "semantic-correct"],
    "captcha": ["verification-tf-serving"],
    "nlp": ["bert-service-tf"],
    "ocr_seal": ["ocr-seal-tf-serving"]
}

tfserver_image_name = {
    "ocr_standard": "tfserving",
    "captcha": "tfserving",
    "nlp": "tfserving",
    "ocr_seal": "tfserving"
}

# service_module_relation = {
#     "commander": [x for x in range(0, 9)],
#     "usercenter": [10],
#     "dataservice": [81],
#     "mage": [x for x in range(31, 51)],
#     # "wulai": [x for x in range(11, 31)],
#     "chatbot": [x for x in range(11, 31)],
#     "nlp": [33, 34, 35],
#     "captcha": [32],
#     "ocr_text_table": [36, 37],
#     "ocr_3rd": [x for x in range(51, 80)],
#     "ocr_seal": [38, 39],
#     "ocr": [40],
#     "ocr_idcard_server": [41, 42]
# }
service_module_relation = {
    # "entcmd": [x for x in range(0, 9)],
    # "entuc": [10],
    "entcmd": [10],
    "entuc": [0],
    "notice": [6],
    "dataservice": [16],
    "mage": [x for x in range(31)],
    # "wulai": [x for x in range(11, 31)],
    "chatbot": [x for x in range(11)],
    "nlp": [34],
    "captcha": [32],
    "ocr_text": [36, 37],
    "ocr_table": [38, 39],
    "ocr_seal": [40],
    "ocr_idcard": [42],
    "ocr": [30],
    "ocr_3rd": [x for x in range(61, 104)]
}

machine_module_relation = {
    "cpu": [x for x in range(34, 129) if x % 2 == 0],
    "gpu": [x for x in range(34, 129) if x % 2 != 0]
}
ocr_standard = range(36, 51)
ocr = range(51, 80)
if DEPLOY_LANGUAGE == "cn":
    module_info = {
        0: ("entuc", "usercenter用户中心"),
        1: ("license", "License授权服务"),
        5: ("rpa-collaboration", "人机协同中心"),
        6: ("notice", "通知中心"),
        10: ("entcmd", "Enterprise Commander"),
        11: ("chatbot", "ChatBot吾来对话机器人"),
        16: ("dataservice", "数据服务"),
        30: ("ocr", "ocr go服务"),
        31: ("mage", "UiBot Mage"),
        32: ("captcha", "验证码识别"),
        33: ("geo", "标准地址"),
        34: ("nlp", "文本分类、信息抽取"),
        36: ("ocr_text_cpu", "通用文字识别(标准版 CPU)"),
        37: ("ocr_text", "通用文字识别(标准版 GPU)"),
        38: ("ocr_table_cpu", "通用表格识别(标准版 CPU)"),
        39: ("ocr_table", "通用表格识别(标准版 GPU)"),
        40: ("ocr_seal_cpu", "印章识别(标准版 CPU)"),
        41: ("ocr_seal", "印章识别(标准版 GPU)"),
        42: ("ocr_idcard_cpu", "通用卡证识别(标准版CPU)"),
        43: ("ocr_idcard", "通用卡证识别(标准版GPU)"),
        45: ("ocr_layout_analysis", "版面分析CPU"),
        47: ("ocr_bizlicense", "营业执照GPU"),
        49: ("ocr_vehlicense", "机动车驾驶证"),
        52: ("nlp_layoutlm", "文档分类模型(标准版CPU)"),
        61: ("i_document", "通用文字识别(合合CPU)"),
        62: ("i_document_gpu", "通用文字识别(合合GPU)"),
        63: ("i_table", "通用表格识别(合合CPU)"),
        64: ("i_table_gpu", "通用表格识别(合合GPU)"),
        65: ("i_receipt", "通用多票据识别(合合CPU)"),
        66: ("i_receipt_gpu", "通用多票据识别(合合GPU)"),
        67: ("i_idcard", "通用卡证识别-身份证(合合CPU)"),
        68: ("i_idcard_gpu", "通用卡证识别-身份证(合合GPU)"),
        69: ("i_bankcard", "通用卡证识别-银行卡(合合CPU)"),
        70: ("i_bankcard_gpu", "通用卡证识别-银行卡(合合GPU)"),
        71: ("i_vehicle", "通用卡证识别-机动车登记证(合合CPU)"),
        72: ("i_vehicle_gpu", "通用卡证识别-机动车登记证(合合GPU)"),
        73: ("i_vehiclelicense", "通用卡证识别-机动车行驶证(合合CPU)"),
        74: ("i_vehiclelicense_gpu", "通用卡证识别-机动车行驶证(合合GPU)"),
        75: ("i_biz", "通用卡证识别-营业执照(合合CPU)"),
        76: ("i_biz_gpu", "通用卡证识别-营业执照(合合GPU)"),
        83: ("i_driverlicense", "通用卡证识别-驾驶证(合合CPU)"),
        84: ("i_driverlicense_gpu", "通用卡证识别-驾驶证(合合GPU)"),
        91: ("i_passport", "通用卡证识别-护照(合合CPU)"),
        92: ("i_passport_gpu", "通用卡证识别-护照(合合GPU)"),
        103: ("i_vehiclecert", "通用卡证识别-车辆合格证(合合CPU)"),
        104: ("i_vehiclecert_gpu", "通用卡证识别-车辆合格证(合合GPU)"),
        112: ("t_document_gpu", "通用文字识别 - 探智立方GPU"),
        114: ("t_table_gpu", "通用表格识别 - 探智立方GPU"),
        116: ("t_receipt_gpu", "通用多票据识别 - 探智立方GPU"),
        118: ("t_idcard_gpu", "通用卡证识别-身份证- 探智立方GPU"),
        120: ("t_biz_gpu", "通用卡证识别-营业执照-探智立方GPU"),
        122: ("t_driverlicense_gpu", "通用卡证识别-驾驶证-探智立方GPU"),
        124: ("t_vehiclelicense_gpu", "通用卡证识别-行驶证-探智立方GPU"),
        126: ("t_bankcard_gpu", "通用卡证识别-银行卡-探智立方GPU"),
        128: ("t_vehicle_gpu", "通用卡证识别-车辆登记证-探智立方GPU"),
        130: ("t_marriage_gpu", "通用卡证识别-结婚证-探智立方GPU"),
        132: ("t_hrb_gpu", "通用卡证识别-户口本-探智立方GPU"),
    }
else:
    module_info = {
        0: ("entuc", "Enterprise Usercenter"),
        1: ("license", "License Service"),
        5: ("rpa-collaboration", "RPA Collaboration"),
        6: ("notice", "Notice Center"),
        10: ("entcmd", "Enterprise Commander"),
        11: ("chatbot", "Conversational AI Platform"),
        16: ("dataservice", "Data Service"),
        30: ("ocr", "Ocr Go"),
        31: ("mage", "Intelligent Document Processing"),
        32: ("captcha", "Captcha"),
        33: ("geo", "Geo"),
        34: ("nlp", "Information Extraction Text Classification"),
        36: ("ocr_text_cpu", "Text REcognition(Standard Edition-CPU)"),
        37: ("ocr_text", "Text REcognition(Standard Edition-GPU)"),
        38: ("ocr_table_cpu", "Form Recognition（Standard Edition-CPU）"),
        39: ("ocr_table", "Form Recognition（Standard Edition-GPU）"),
        40: ("ocr_seal_cpu", "Seal Recognition(Standard Edition-CPU)"),
        41: ("ocr_seal", "Seal Recognition(Standard Edition-GPU)"),
        42: ("ocr_idcard_cpu", "Id Card Recognition(Standard Edition-CPU)"),
        43: ("ocr_idcard", "Id Card Recognition(Standard Edition-GPU)"),
        45: ("ocr_layout_analysis", "Layout Analysis CPU"),
        47: ("ocr_bizlicense", "ocr bizlicense GPU"),
        49: ("ocr_vehlicense", "tor vehicle driving license GPU"),
        52: ("nlp_layoutlm", "Document classification model(Standard Edition-CPU)"),
        61: ("i_document", "Text REcognition(hehe CPU)"),
        62: ("i_document_gpu", "Text REcognition(hehe CPU)"),
        63: ("i_table", "Table REcognition(hehe CPU)"),
        64: ("i_table_gpu", "Table REcognition(hehe GPU)"),
        65: ("i_receipt", "Receipt REcognition(hehe CPU)"),
        66: ("i_receipt_gpu", "Receipt REcognition(hehe GPU)"),
        67: ("i_idcard", "IdCard REcognition(hehe CPU)"),
        68: ("i_idcard_gpu", "IdCard REcognition(hehe GPU)"),
        69: ("i_bankcard", "BankCard REcognition(hehe CPU)"),
        70: ("i_bankcard_gpu", "BankCard REcognition(hehe GPU)"),
        71: ("i_vehicle", "Vehicle REcognition(hehe CPU)"),
        72: ("i_vehicle_gpu", "Vehicle REcognition(hehe GPU)"),
        73: ("i_vehiclelicense", "Vehiclelicense REcognition(hehe CPU)"),
        74: ("i_vehiclelicense_gpu", "Vehiclelicense REcognition(hehe GPU)"),
        75: ("i_biz", "Biz REcognition(hehe GPU)"),
        76: ("i_biz_gpu", "Biz REcognition(hehe GPU)"),
        83: ("i_driverlicense", "Driver REcognition(hehe GPU)"),
        84: ("i_driverlicense_gpu", "Driver REcognition(hehe GPU)"),
        91: ("i_passport", "Passport REcognition(hehe GPU)"),
        92: ("i_passport_gpu", "Passport REcognition(hehe GPU)"),
        103: ("i_vehiclecert", "Vehiclecert REcognition(hehe GPU)"),
        104: ("i_vehiclecert_gpu", "Vehiclecert REcognition(hehe GPU)"),
        112: ("t_document_gpu", "Text REcognition(Tanzhilifang GPU)"),
        114: ("t_table_gpu", "Table REcognition(Tanzhilifang GPU)"),
        116: ("t_receipt_gpu", "Receipt REcognition(Tanzhilifang GPU)"),
        118: ("t_idcard_gpu", "IDCard REcognition(Tanzhilifang GPU)"),
        120: ("t_biz_gpu", "Biz REcognition(Tanzhilifang GPU)"),
        122: ("t_driverlicense_gpu", "Driverlicense REcognition(Tanzhilifang GPU)"),
        124: ("t_vehiclelicense_gpu", "Vehiclelicense REcognition(Tanzhilifang GPU)"),
        126: ("t_bankcard_gpu", "BankCard REcognition(Tanzhilifang GPU)"),
        128: ("t_vehicle_gpu", "Vehicle REcognition(Tanzhilifang GPU)"),
        130: ("t_marriage_gpu", "Marriage REcognition(Tanzhilifang GPU)"),
        132: ("t_hrb_gpu", "Hrb REcognition(Tanzhilifang GPU)"),
    }
# module_info = {
#     0: ("commander", "UiBot Commander"),
#     10: ("usercenter", "usercenter用户中心"),
#     # 11: ("wulai", "吾来对话机器人"),
#     11: ("chatbot", "ChatBot吾来对话机器人"),
#     31: ("mage", "UiBot Mage"),
#     32: ("captcha", "验证码识别"),
#     33: ("geo", "标准地址"),
#     34: ("nlp", "文本分类、信息抽取"),
#     35: ("nlp", "文本分类、信息抽取(GPU)"),
#     36: ("ocr_text_table", "通用文字识别(标准版 CPU)"),
#     37: ("ocr_text_table_gpu", "通用文字识别(标准版 GPU)"),
#     38: ("ocr_seal", "印章识别(标准版 CPU)"),
#     39: ("ocr_seal_gpu", "印章识别(标准版 GPU)"),
#     40: ("ocr", "ocr-go服务-cpu"),
#     41: ("ocr", "ocr-go服务-gpu"),
#     42: ("ocr_idcard_server", "通用卡证识别(标准版)CPU"),
#     43: ("ocr_idcard_server_gpu", "通用卡证识别(标准版)GPU"),
#     51: ("ocr_document_gpu", "通用文字识别(高级版 GPU)"),
#     52: ("ocr_document", "通用文字识别(高级版 CPU)"),
#     53: ("ocr_table_gpu", "通用表格识别(高级版 GPU)"),
#     54: ("ocr_table", "通用表格识别(高级版 CPU)"),
#     55: ("ocr_receipt_gpu", "通用票据识别(高级版 GPU)"),
#     56: ("ocr_receipt", "通用票据识别(高级版 CPU)"),
#     57: ("ocr_idcard_gpu", "通用卡证识别-身份证(高级版 GPU)"),
#     58: ("ocr_idcard", "通用卡证识别-身份证(高级版 CPU)"),
#     59: ("ocr_bankcard_gpu", "通用卡证识别-银行卡(高级版 GPU)"),
#     60: ("ocr_bankcard", "通用卡证识别-银行卡(高级版 CPU)"),
#     61: ("ocr_vehicle_gpu", "通用卡证识别-机动车登记证(高级版 GPU)"),
#     62: ("ocr_vehicle", "通用卡证识别-机动车登记证(高级版 CPU)"),
#     63: ("ocr_vehiclelicense_gpu", "通用卡证识别-机动车行驶证(高级版 GPU)"),
#     64: ("ocr_vehiclelicense", "通用卡证识别-机动车行驶证(高级版 CPU)"),
#     65: ("ocr_biz_gpu", "通用卡证识别-营业执照(高级版 GPU)"),
#     66: ("ocr_biz", "通用卡证识别-营业执照(高级版 CPU)"),
#     81: ("dataservice", "数据服务")
# }
menu_relation = {
    1001: {
        36: 100101,
        37: 100102,
        51: 100103,
        52: 100104
    }
}

middleware_port_relation = {
    "minio": {
        "port": 9000,
        "nginx_proxy_port": 10000
    },
    "harbor": {
        "http_port": 8888,
        "nginx_harbor_proxy_port": 8889
    },
    "redis": {
        "port": 6379,
        "port_sentinel": 26379
    },
    "nginx": {
        "k8s_proxy_port": 6444,
        "entcmd_proxy_port": 8080,
        "commander_tenant_port": 8081,
        "mage_proxy_port": 8082,
        "chatbot_proxy_port": 8083,
        "entuc_proxy_port": 8084,
        "dataservice_proxy_port": 8085,
        "notice_proxy_port": 8089,
        "license_web_proxy_port": 8090,
        "rpa_collaboration_backend_proxy_port": 8091,
        "creativity_proxy_port": 8092,
        "entfs_proxy_port": 8093,
        "entiap_proxy_port": 8094
    },
    "mysql": {
        "port": 3306,
        "proxysql_cluster_port": 6032,
        "proxysql_port": 6033,
        "nginx_proxy_port": 6034
    },
    "elasticsearch": {
        "http_port": 9200,
        "tcp_port": 9300
    },
    "rabbitmq": {
        "port": 5672,
        "manage_port": 15672,
        "empd_port": 4369,
        "erlang_port": 25672
    },
    "etcd": {
        "http_port": 12379,
        "tcp_port": 12380
    },
    "identity": {
        "port": 6060,
        "nginx_proxy_port": 6061
    },
    "monitor": {
        "grafana_port": 3000,
        "prometheus_port": 9090,
        "mysql_exporter_port": 9104,
        "redis_exporter_port": 9121,
        "rabbitmq_exporter_port": 9419,
        "elasticsearch_exporter_port": 9114,
        "node_exporter_port": 9100,
        "k8s_metrics_port": 31388,
        "istio_prometheus_port": 31390
    },
    "siber": {
        "port": 88
    },
    "ocr": {
        "ocr_document_gpu": 30006,
        "ocr_table_gpu": 30007,
        "ocr_receipt_gpu": 30008,
        "ocr_idcard_gpu": 30009,
        "ocr_bankcard_gpu": 30013,
        "ocr_vehicle_gpu": 30010,
        "ocr_vehiclelicense_gpu": 30011,
        "ocr_biz_gpu": 30012,
        "ocr_passport_gpu": 30014
    },
    "chronyd": {
        "port": 123,
        "cmdport": 323
    },
}

def init_port_config():
    conf = YamlConfig(PORT_MAP, data=middleware_port_relation)
    conf.write_file(backup=False)

def find_module_by_key(module: str) -> list:
    l = []
    for id in module_info:
        if module_info[id][0] == module:
            res = {}
            res['id'] = id
            res['module'] = module
            res['description'] = module_info[id][1]
            l.append(res)
    return l

def get_module_description(module_id: int) -> str:
    return module_info[module_id][1]

def get_module_key(module_id: int) -> str:
    return module_info[module_id][0]

def get_module_keys(module_ids: list) -> list:
    return [get_module_key(x) for x in module_ids]

def get_all_ports(middleware="") -> list:
    ''':return list[str]'''
    l = []
    try:
        relation = YamlConfig(PORT_MAP).read_file()
    except Exception:
        relation = middleware_port_relation
    if middleware:
        try:
            for k in relation[middleware]:
                l.append(str(relation[middleware][k]))
        except KeyError:
            pass
    else:
        for mid in relation:
            for k in relation[mid]:
                l.append(str(relation[mid][k]))
    return l

def get_port(middleware: str, key: str) -> int:
    try:
        relation = YamlConfig(PORT_MAP).read_file()
    except Exception:
        relation = middleware_port_relation

    try:
        return relation[middleware][key]
    except KeyError:
        for k in relation[middleware].keys():
            if key in k:
                return relation[middleware][k]
        return 0
